@@ -21,25 +21,17 @@
 
 #include "abstractfile/indexercommon.h"
 
-#include "musicaudiotrack.h"
-#include "notificationitem.h"
 #include "filescanner.h"
 
-#if defined KF5FileMetaData_FOUND && KF5FileMetaData_FOUND
-#include <KFileMetaData/EmbeddedImageData>
-#endif
-
 #include <QThread>
 #include <QHash>
 #include <QFileInfo>
 #include <QFile>
 #include <QDir>
 #include <QFileSystemWatcher>
-#include <QMimeDatabase>
 #include <QSet>
 #include <QPair>
 #include <QAtomicInt>
-#include <QDebug>
 
 #include <QtGlobal>
 
@@ -60,20 +52,12 @@ public:
 
     FileScanner mFileScanner;
 
-    QMimeDatabase mMimeDb;
-
-#if defined KF5FileMetaData_FOUND && KF5FileMetaData_FOUND
-    KFileMetaData::EmbeddedImageData mImageScanner;
-#endif
-
     QHash<QUrl, QDateTime> mAllFiles;
 
     QAtomicInt mStopRequest = 0;
 
     int mImportedTracksCount = 0;
 
-    int mNotificationUpdateInterval = 1;
-
     int mNewFilesEmitInterval = 1;
 
     bool mHandleNewFiles = true;
@@ -100,7 +84,7 @@ void AbstractFileListing::init()
     Q_EMIT askRestoredTracks();
 }
 
-void AbstractFileListing::newTrackFile(const MusicAudioTrack &partialTrack)
+void AbstractFileListing::newTrackFile(const DataTypes::TrackDataType &partialTrack)
 {
     auto scanFileInfo = QFileInfo(partialTrack.resourceURI().toLocalFile());
     const auto &newTrack = scanOneFile(partialTrack.resourceURI(), scanFileInfo);
@@ -119,11 +103,18 @@ void AbstractFileListing::restoredTracks(QHash<QUrl, QDateTime> allFiles)
 
 void AbstractFileListing::setAllRootPaths(const QStringList &allRootPaths)
 {
-    if (d->mAllRootPaths == allRootPaths) {
-        return;
+    //resolve symlinks
+    QStringList allPaths;
+    for (const auto &path : allRootPaths) {
+        QFileInfo newPath(path);
+        if (newPath.isSymLink()) {
+            allPaths << newPath.symLinkTarget();
+        } else {
+            allPaths << path;
+        }
     }
 
-    d->mAllRootPaths = allRootPaths;
+    d->mAllRootPaths = allPaths;
 }
 
 void AbstractFileListing::databaseFinishedInsertingTracksList()
@@ -148,7 +139,7 @@ const QStringList &AbstractFileListing::allRootPaths() const
     return d->mAllRootPaths;
 }
 
-void AbstractFileListing::scanDirectory(QList<MusicAudioTrack> &newFiles, const QUrl &path)
+void AbstractFileListing::scanDirectory(DataTypes::ListTrackDataType &newFiles, const QUrl &path)
 {
     if (d->mStopRequest == 1) {
         return;
@@ -239,15 +230,14 @@ void AbstractFileListing::scanDirectory(QList<MusicAudioTrack> &newFiles, const
             newFiles.push_back(newTrack);
 
             ++d->mImportedTracksCount;
-            if (d->mImportedTracksCount % d->mNotificationUpdateInterval == 0) {
-                d->mNotificationUpdateInterval = std::min(50, 1 + d->mNotificationUpdateInterval * 2);
-            }
 
             if (newFiles.size() > d->mNewFilesEmitInterval && d->mStopRequest == 0) {
                 d->mNewFilesEmitInterval = std::min(50, 1 + d->mNewFilesEmitInterval * d->mNewFilesEmitInterval);
                 emitNewFiles(newFiles);
                 newFiles.clear();
             }
+        } else {
+            qCDebug(orgKdeElisaIndexer()) << "AbstractFileListing::scanDirectory" << newFilePath << "is not a valid track";
         }
 
         if (d->mStopRequest == 1) {
@@ -297,34 +287,33 @@ void AbstractFileListing::refreshContent()
     triggerRefreshOfContent();
 }
 
-MusicAudioTrack AbstractFileListing::scanOneFile(const QUrl &scanFile, const QFileInfo &scanFileInfo)
+DataTypes::TrackDataType AbstractFileListing::scanOneFile(const QUrl &scanFile, const QFileInfo &scanFileInfo)
 {
-    MusicAudioTrack newTrack;
+    DataTypes::TrackDataType newTrack;
 
     qCDebug(orgKdeElisaIndexer) << "AbstractFileListing::scanOneFile" << scanFile;
 
     auto localFileName = scanFile.toLocalFile();
 
-    const auto &fileMimeType = d->mMimeDb.mimeTypeForFile(localFileName);
-    if (!fileMimeType.name().startsWith(QStringLiteral("audio/"))) {
+    if (!d->mFileScanner.shouldScanFile(localFileName)) {
         return newTrack;
     }
 
     if (scanFileInfo.exists()) {
         auto itExistingFile = d->mAllFiles.find(scanFile);
         if (itExistingFile != d->mAllFiles.end()) {
-            if (*itExistingFile >= scanFileInfo.fileTime(QFile::FileModificationTime)) {
+            if (*itExistingFile >= scanFileInfo.metadataChangeTime()) {
                 d->mAllFiles.erase(itExistingFile);
                 return newTrack;
             }
         }
     }
 
-    newTrack = d->mFileScanner.scanOneFile(scanFile, d->mMimeDb);
+    newTrack = d->mFileScanner.scanOneFile(scanFile);
 
     if (newTrack.isValid()) {
-        newTrack.setHasEmbeddedCover(checkEmbeddedCoverImage(localFileName));
-        newTrack.setFileModificationTime(scanFileInfo.fileTime(QFile::FileModificationTime));
+        newTrack[DataTypes::HasEmbeddedCover] = checkEmbeddedCoverImage(localFileName);
+        newTrack[DataTypes::FileModificationTime] = scanFileInfo.metadataChangeTime();
 
         if (scanFileInfo.exists()) {
             watchPath(scanFile.toLocalFile());
@@ -374,7 +363,9 @@ void AbstractFileListing::addFileInDirectory(const QUrl &newFile, const QUrl &di
 
 void AbstractFileListing::scanDirectoryTree(const QString &path)
 {
-    auto newFiles = QList<MusicAudioTrack>();
+    auto newFiles = DataTypes::ListTrackDataType();
+
+    qCDebug(orgKdeElisaIndexer()) << "AbstractFileListing::scanDirectoryTree" << path;
 
     scanDirectory(newFiles, QUrl::fromLocalFile(path));
 
@@ -388,36 +379,21 @@ void AbstractFileListing::setHandleNewFiles(bool handleThem)
     d->mHandleNewFiles = handleThem;
 }
 
-void AbstractFileListing::emitNewFiles(const QList<MusicAudioTrack> &tracks)
+void AbstractFileListing::emitNewFiles(const DataTypes::ListTrackDataType &tracks)
 {
     Q_EMIT tracksList(tracks, d->mAllAlbumCover);
 }
 
-void AbstractFileListing::addCover(const MusicAudioTrack &newTrack)
+void AbstractFileListing::addCover(const DataTypes::TrackDataType &newTrack)
 {
-    auto itCover = d->mAllAlbumCover.find(newTrack.albumName());
+    auto itCover = d->mAllAlbumCover.find(newTrack.album());
     if (itCover != d->mAllAlbumCover.end()) {
         return;
     }
 
-    QFileInfo trackFilePath(newTrack.resourceURI().toLocalFile());
-    QDir trackFileDir = trackFilePath.absoluteDir();
-    QString dirNamePattern = QStringLiteral("*") + trackFileDir.dirName() + QStringLiteral("*");
-    QStringList filters;
-    filters << QStringLiteral("*[Cc]over*.jpg") << QStringLiteral("*[Cc]over*.png")
-            << QStringLiteral("*[Ff]older*.jpg") << QStringLiteral("*[Ff]older*.png")
-            << QStringLiteral("*[Ff]ront*.jpg") << QStringLiteral("*[Ff]ront*.png")
-            << dirNamePattern + QStringLiteral(".jpg") << dirNamePattern + QStringLiteral(".png")
-            << dirNamePattern.toLower() + QStringLiteral(".jpg") << dirNamePattern.toLower() + QStringLiteral(".png");
-    dirNamePattern.remove(QLatin1Char(' '));
-    filters << dirNamePattern + QStringLiteral(".jpg") << dirNamePattern + QStringLiteral(".png")
-            << dirNamePattern.toLower() + QStringLiteral(".jpg") << dirNamePattern.toLower() + QStringLiteral(".png");
-    trackFileDir.setNameFilters(filters);
-    QFileInfoList coverFiles = trackFileDir.entryInfoList();
-    if (coverFiles.isEmpty()) {
-        return;
-    } else {
-        d->mAllAlbumCover[newTrack.resourceURI().toString()] = QUrl::fromLocalFile(coverFiles.at(0).absoluteFilePath());
+    auto coverUrl = d->mFileScanner.searchForCoverFile(newTrack.resourceURI().toLocalFile());
+    if (!coverUrl.isEmpty()) {
+        d->mAllAlbumCover[newTrack.resourceURI().toString()] = coverUrl;
     }
 }
 
@@ -463,6 +439,8 @@ void AbstractFileListing::checkFilesToRemove()
         allRemovedFiles.push_back(itFile.key());
     }
 
+    qCDebug(orgKdeElisaIndexer()) << "AbstractFileListing::checkFilesToRemove" << allRemovedFiles.size();
+
     if (!allRemovedFiles.isEmpty()) {
         setWaitEndTrackRemoval(true);
         Q_EMIT removedTracksList(allRemovedFiles);
@@ -476,17 +454,7 @@ FileScanner &AbstractFileListing::fileScanner()
 
 bool AbstractFileListing::checkEmbeddedCoverImage(const QString &localFileName)
 {
-#if defined KF5FileMetaData_FOUND && KF5FileMetaData_FOUND
-    auto imageData = d->mImageScanner.imageData(localFileName);
-
-    if (imageData.contains(KFileMetaData::EmbeddedImageData::FrontCover)) {
-        if (!imageData[KFileMetaData::EmbeddedImageData::FrontCover].isEmpty()) {
-            return true;
-        }
-    }
-#endif
-
-    return false;
+    return d->mFileScanner.checkEmbeddedCoverImage(localFileName);
 }
 
 bool AbstractFileListing::waitEndTrackRemoval() const
@@ -499,10 +467,5 @@ void AbstractFileListing::setWaitEndTrackRemoval(bool wait)
     d->mWaitEndTrackRemoval = wait;
 }
 
-const QMimeDatabase &AbstractFileListing::mimeDatabase() const
-{
-    return d->mMimeDb;
-}
-
 
 #include "moc_abstractfilelisting.cpp"
