@@ -40,11 +40,11 @@
 #include "managemediaplayercontrol.h"
 #include "manageheaderbar.h"
 #include "manageaudioplayer.h"
-#include "musicaudiotrack.h"
 #include "musiclistenersmanager.h"
 #include "trackslistener.h"
 #include "viewmanager.h"
 #include "databaseinterface.h"
+#include "datatypes.h"
 #include "models/datamodel.h"
 #include "models/trackmetadatamodel.h"
 #include "models/trackcontextmetadatamodel.h"
@@ -63,10 +63,6 @@
 #endif
 
 #include "audiowrapper.h"
-#include "notificationitem.h"
-#include "topnotificationmanager.h"
-#include "elisautils.h"
-#include "datatype.h"
 
 #if defined Qt5DBus_FOUND && Qt5DBus_FOUND
 #include "mpris2/mpris2.h"
@@ -113,7 +109,6 @@ void ElisaQmlTestPlugin::registerTypes(const char *uri)
     qRegisterMetaType<UpnpContentDirectoryModel*>();
     qRegisterMetaType<UpnpDeviceDescription*>();
 #endif
-    qRegisterMetaType<DataUtils::DataType>("DataUtils::DataType");
 
     qmlRegisterType<MediaPlayList>(uri, 1, 0, "MediaPlayList");
     qmlRegisterType<ManageMediaPlayerControl>(uri, 1, 0, "ManageMediaPlayerControl");
@@ -136,8 +131,6 @@ void ElisaQmlTestPlugin::registerTypes(const char *uri)
 #endif
 
     qmlRegisterType<AudioWrapper>(uri, 1, 0, "AudioWrapper");
-    qmlRegisterType<TopNotificationManager>(uri, 1, 0, "TopNotificationManager");
-    qmlRegisterUncreatableMetaObject(DataUtils::staticMetaObject, uri, 1, 0, "DataUtils", QStringLiteral("Only enums"));
     qmlRegisterUncreatableType<DatabaseInterface>(uri, 1, 0, "DatabaseInterface", QStringLiteral("Only created in c++"));
 
 #if defined Qt5DBus_FOUND && Qt5DBus_FOUND
@@ -148,15 +141,15 @@ void ElisaQmlTestPlugin::registerTypes(const char *uri)
     qRegisterMetaType<AbstractMediaProxyModel*>();
     qRegisterMetaType<QHash<QString,QUrl>>("QHash<QString,QUrl>");
     qRegisterMetaType<QHash<QUrl,QDateTime>>("QHash<QUrl,QDateTime>");
-    qRegisterMetaType<QList<MusicAudioTrack>>("QList<MusicAudioTrack>");
-    qRegisterMetaType<QList<MusicAudioTrack>>("QVector<MusicAudioTrack>");
     qRegisterMetaType<QVector<qulonglong>>("QVector<qulonglong>");
     qRegisterMetaType<QHash<qulonglong,int>>("QHash<qulonglong,int>");
-    qRegisterMetaType<DatabaseInterface::ListTrackDataType>("DatabaseInterface::ListTrackDataType");
-    qRegisterMetaType<DatabaseInterface::ListAlbumDataType>("DatabaseInterface::ListAlbumDataType");
-    qRegisterMetaType<DatabaseInterface::ListArtistDataType>("DatabaseInterface::ListArtistDataType");
-    qRegisterMetaType<DatabaseInterface::ListGenreDataType>("DatabaseInterface::ListGenreDataType");
+    qRegisterMetaType<DataTypes::ListTrackDataType>("DataTypes::ListTrackDataType");
+    qRegisterMetaType<DataTypes::ListRadioDataType>("DataTypes::ListRadioDataType");
+    qRegisterMetaType<DataTypes::ListAlbumDataType>("DataTypes::ListAlbumDataType");
+    qRegisterMetaType<DataTypes::ListArtistDataType>("DataTypes::ListArtistDataType");
+    qRegisterMetaType<DataTypes::ListGenreDataType>("DataTypes::ListGenreDataType");
     qRegisterMetaType<ModelDataLoader::ListTrackDataType>("ModelDataLoader::ListTrackDataType");
+    qRegisterMetaType<ModelDataLoader::ListRadioDataType>("ModelDataLoader::ListRadioDataType");
     qRegisterMetaType<ModelDataLoader::ListAlbumDataType>("ModelDataLoader::ListAlbumDataType");
     qRegisterMetaType<ModelDataLoader::ListArtistDataType>("ModelDataLoader::ListArtistDataType");
     qRegisterMetaType<ModelDataLoader::ListGenreDataType>("ModelDataLoader::ListGenreDataType");
@@ -164,22 +157,21 @@ void ElisaQmlTestPlugin::registerTypes(const char *uri)
     qRegisterMetaType<TracksListener::ListTrackDataType>("TracksListener::ListTrackDataType");
     qRegisterMetaType<QMap<QString, int>>();
     qRegisterMetaType<QAction*>();
-    qRegisterMetaType<NotificationItem>("NotificationItem");
     qRegisterMetaType<QMap<QString,int>>("QMap<QString,int>");
-    qRegisterMetaType<ElisaUtils::PlayListEnqueueMode>("PlayListEnqueueMode");
-    qRegisterMetaType<ElisaUtils::PlayListEnqueueTriggerPlay>("PlayListEnqueueTriggerPlay");
-    qRegisterMetaType<ElisaUtils::PlayListEntryType>("PlayListEntryType");
-    qRegisterMetaType<ElisaUtils::EntryData>("EntryData");
-    qRegisterMetaType<ElisaUtils::EntryDataList>("EntryDataList");
-    qRegisterMetaType<DatabaseInterface::TrackDataType>("DatabaseInterface::TrackDataType");
-    qRegisterMetaType<DatabaseInterface::AlbumDataType>("DatabaseInterface::AlbumDataType");
-    qRegisterMetaType<DatabaseInterface::ArtistDataType>("DatabaseInterface::ArtistDataType");
-    qRegisterMetaType<DatabaseInterface::GenreDataType>("DatabaseInterface::GenreDataType");
+    qRegisterMetaType<ElisaUtils::PlayListEnqueueMode>("ElisaUtils::PlayListEnqueueMode");
+    qRegisterMetaType<ElisaUtils::PlayListEnqueueTriggerPlay>("ElisaUtils::PlayListEnqueueTriggerPlay");
+    qRegisterMetaType<ElisaUtils::PlayListEntryType>("ElisaUtils::PlayListEntryType");
+    qRegisterMetaType<ElisaUtils::EntryData>("ElisaUtils::EntryData");
+    qRegisterMetaType<ElisaUtils::EntryDataList>("ElisaUtils::EntryDataList");
+    qRegisterMetaType<ElisaUtils::FilterType>("ElisaUtils::FilterType");
+    qRegisterMetaType<DataTypes::TrackDataType>("DataTypes::TrackDataType");
+    qRegisterMetaType<DataTypes::AlbumDataType>("DataTypes::AlbumDataType");
+    qRegisterMetaType<DataTypes::ArtistDataType>("DataTypes::ArtistDataType");
+    qRegisterMetaType<DataTypes::GenreDataType>("DataTypes::GenreDataType");
     qRegisterMetaType<ModelDataLoader::TrackDataType>("ModelDataLoader::TrackDataType");
     qRegisterMetaType<TracksListener::TrackDataType>("TracksListener::TrackDataType");
     qRegisterMetaType<ViewManager::ViewsType>("ViewManager::ViewsType");
-
-    qRegisterMetaTypeStreamOperators<ManageMediaPlayerControl::PlayerState>("PlayListControler::PlayerState");
+    qRegisterMetaType<ViewManager::SortOrder>("ViewManager::SortOrder");
 
     qmlRegisterUncreatableType<ElisaApplication>(uri, 1, 0, "ElisaApplication", QStringLiteral("only one and done in c++"));
     qmlRegisterUncreatableMetaObject(ElisaUtils::staticMetaObject, uri, 1, 0, "ElisaUtils", QStringLiteral("Namespace ElisaUtils"));
