@@ -17,14 +17,13 @@
 
 #include "localfilelisting.h"
 
-#include "musicaudiotrack.h"
+#include "abstractfile/indexercommon.h"
 
 #include <QThread>
 #include <QHash>
 #include <QFileInfo>
 #include <QDir>
 #include <QFileSystemWatcher>
-#include <QMimeDatabase>
 #include <QStandardPaths>
 
 #include <QDebug>
@@ -46,6 +45,7 @@ LocalFileListing::~LocalFileListing()
 
 void LocalFileListing::executeInit(QHash<QUrl, QDateTime> allFiles)
 {
+    qCDebug(orgKdeElisaIndexer()) << "LocalFileListing::executeInit" << "with" << allFiles.size() << "files";
     AbstractFileListing::executeInit(std::move(allFiles));
 }
 
@@ -53,6 +53,8 @@ void LocalFileListing::triggerRefreshOfContent()
 {
     Q_EMIT indexingStarted();
 
+    qCDebug(orgKdeElisaIndexer()) << "LocalFileListing::triggerRefreshOfContent" << allRootPaths();
+
     AbstractFileListing::triggerRefreshOfContent();
 
     const auto &rootPaths = allRootPaths();
