@@ -88,11 +88,14 @@ int main(int argc, char *argv[])
 #endif
 
     QApplication::setAttribute(Qt::AA_UseHighDpiPixmaps);
+    QApplication::setAttribute(Qt::AA_EnableHighDpiScaling);
 
     qputenv("QT_GSTREAMER_USE_PLAYBIN_VOLUME", "true");
 
     QApplication app(argc, argv);
 
+    KLocalizedString::setApplicationDomain("elisa");
+
 #if defined Qt5AndroidExtras_FOUND && Qt5AndroidExtras_FOUND
     qInfo() << QCoreApplication::arguments();
 
@@ -108,8 +111,6 @@ int main(int argc, char *argv[])
 
     QApplication::setWindowIcon(QIcon::fromTheme(QStringLiteral("elisa")));
 
-    KLocalizedString::setApplicationDomain("elisa");
-
     KAboutData aboutData( QStringLiteral("elisa"),
                           i18n("Elisa"),
                           QStringLiteral(ELISA_VERSION_STRING),
@@ -127,8 +128,6 @@ int main(int argc, char *argv[])
 
     KAboutData::setApplicationData(aboutData);
 
-    KLocalizedString::setApplicationDomain("elisa");
-
     QCommandLineParser parser;
     parser.addHelpOption();
     parser.addVersionOption();
@@ -151,7 +150,6 @@ int main(int argc, char *argv[])
 #endif
 
     engine.rootContext()->setContextObject(new KLocalizedContext(&engine));
-    engine.rootContext()->setContextProperty(QStringLiteral("logicalDpi"), QGuiApplication::primaryScreen()->logicalDotsPerInch());
 
 #if defined KF5DBusAddons_FOUND && KF5DBusAddons_FOUND
     KDBusService elisaService(KDBusService::Unique);
@@ -169,7 +167,7 @@ int main(int argc, char *argv[])
     auto realArgumentsList = parser.positionalArguments();
 
     for (auto oneArgument : realArgumentsList) {
-        arguments.push_back(ElisaUtils::EntryData{0, oneArgument});
+        arguments.push_back(ElisaUtils::EntryData{{}, {}, QUrl(oneArgument)});
     }
 
     myApp->setArguments(arguments);
