@@ -413,6 +413,14 @@ void ManageAudioPlayer::setPlayerPosition(qint64 playerPosition)
     QTimer::singleShot(0, [this]() {Q_EMIT playControlPositionChanged();});
 }
 
+void ManageAudioPlayer::setCurrentPlayingForRadios(QString title, QString nowPlaying)
+{
+    if (mPlayListModel && mCurrentTrack.isValid()) {
+        Q_EMIT currentPlayingForRadiosChanged(title, MediaPlayList::TitleRole);
+        Q_EMIT currentPlayingForRadiosChanged(nowPlaying, MediaPlayList::ArtistRole);
+    }
+}
+
 void ManageAudioPlayer::setPlayControlPosition(int playerPosition)
 {
     Q_EMIT seek(playerPosition);
@@ -557,8 +565,9 @@ void ManageAudioPlayer::restorePreviousState()
         return;
     }
 
-    if (*itTitle != mCurrentTrack.data(mTitleRole) || *itArtistName != mCurrentTrack.data(mArtistNameRole) ||
-            *itAlbumName != mCurrentTrack.data(mAlbumNameRole)) {
+    if (*itTitle != mCurrentTrack.data(mTitleRole) ||
+            (itArtistName->isValid() && *itArtistName != mCurrentTrack.data(mArtistNameRole)) ||
+            (itAlbumName->isValid() && *itAlbumName != mCurrentTrack.data(mAlbumNameRole))) {
         if (mCurrentTrack.isValid() && mCurrentTrack.data(mTitleRole).isValid() && mCurrentTrack.data(mArtistNameRole).isValid() &&
                 mCurrentTrack.data(mAlbumNameRole).isValid()) {
             mPersistentState.clear();
