@@ -20,6 +20,8 @@
 
 #include "elisaLib_export.h"
 
+#include "elisautils.h"
+
 #include <QObject>
 #include <QList>
 #include <QPersistentModelIndex>
@@ -36,11 +38,6 @@ class ELISALIB_EXPORT ManageHeaderBar : public QObject
                WRITE setCurrentTrack
                NOTIFY currentTrackChanged)
 
-    Q_PROPERTY(QAbstractItemModel* playListModel
-               READ playListModel
-               WRITE setPlayListModel
-               NOTIFY playListModelChanged)
-
     Q_PROPERTY(int artistRole
                READ artistRole
                WRITE setArtistRole
@@ -76,6 +73,11 @@ class ELISALIB_EXPORT ManageHeaderBar : public QObject
                WRITE setDatabaseIdRole
                NOTIFY databaseIdRoleChanged)
 
+    Q_PROPERTY(int trackTypeRole
+               READ trackTypeRole
+               WRITE setTrackTypeRole
+               NOTIFY trackTypeRoleChanged)
+
     Q_PROPERTY(int albumIdRole
                READ albumIdRole
                WRITE setAlbumIdRole
@@ -114,6 +116,10 @@ class ELISALIB_EXPORT ManageHeaderBar : public QObject
                READ databaseId
                NOTIFY databaseIdChanged)
 
+    Q_PROPERTY(ElisaUtils::PlayListEntryType trackType
+               READ trackType
+               NOTIFY trackTypeChanged)
+
     Q_PROPERTY(qulonglong albumId
                READ albumId
                NOTIFY albumIdChanged)
@@ -122,18 +128,12 @@ class ELISALIB_EXPORT ManageHeaderBar : public QObject
                READ isValid
                NOTIFY isValidChanged)
 
-    Q_PROPERTY(int remainingTracks
-               READ remainingTracks
-               NOTIFY remainingTracksChanged)
-
 public:
 
     explicit ManageHeaderBar(QObject *parent = nullptr);
 
     QPersistentModelIndex currentTrack() const;
 
-    QAbstractItemModel* playListModel() const;
-
     int artistRole() const;
 
     int titleRole() const;
@@ -148,6 +148,8 @@ public:
 
     int databaseIdRole() const;
 
+    int trackTypeRole() const;
+
     int albumIdRole() const;
 
     int isValidRole() const;
@@ -166,18 +168,16 @@ public:
 
     qulonglong databaseId() const;
 
+    ElisaUtils::PlayListEntryType trackType() const;
+
     qulonglong albumId() const;
 
     bool isValid() const;
 
-    int remainingTracks() const;
-
 Q_SIGNALS:
 
     void currentTrackChanged();
 
-    void playListModelChanged();
-
     void artistRoleChanged();
 
     void titleRoleChanged();
@@ -192,6 +192,8 @@ Q_SIGNALS:
 
     void databaseIdRoleChanged();
 
+    void trackTypeRoleChanged();
+
     void albumIdRoleChanged();
 
     void isValidRoleChanged();
@@ -208,20 +210,18 @@ Q_SIGNALS:
 
     void imageChanged();
 
-    void remainingTracksChanged();
-
     void databaseIdChanged();
 
     void albumIdChanged();
 
+    void trackTypeChanged();
+
     void isValidChanged();
 
 public Q_SLOTS:
 
     void setCurrentTrack(const QPersistentModelIndex &currentTrack);
 
-    void setPlayListModel(QAbstractItemModel* aPlayListModel);
-
     void setArtistRole(int value);
 
     void setTitleRole(int value);
@@ -236,22 +236,12 @@ public Q_SLOTS:
 
     void setDatabaseIdRole(int databaseIdRole);
 
+    void setTrackTypeRole(int trackTypeRole);
+
     void setAlbumIdRole(int albumIdRole);
 
     void setIsValidRole(int isValidRole);
 
-    void playListLayoutChanged(const QList<QPersistentModelIndex> &parents, QAbstractItemModel::LayoutChangeHint hint);
-
-    void tracksInserted(const QModelIndex &parent, int first, int last);
-
-    void tracksDataChanged(const QModelIndex &topLeft, const QModelIndex &bottomRight, const QVector<int> &roles);
-
-    void tracksAboutToBeMoved(const QModelIndex &parent, int start, int end, const QModelIndex &destination, int row);
-
-    void tracksMoved(const QModelIndex &parent, int start, int end, const QModelIndex &destination, int row);
-
-    void tracksRemoved(const QModelIndex &parent, int first, int last);
-
 private:
 
     void notifyArtistProperty();
@@ -268,16 +258,14 @@ private:
 
     void notifyDatabaseIdProperty();
 
+    void notifyTrackTypeProperty();
+
     void notifyAlbumIdProperty();
 
     void notifyIsValidProperty();
 
-    void notifyRemainingTracksProperty();
-
     QPersistentModelIndex mCurrentTrack;
 
-    QAbstractItemModel *mPlayListModel = nullptr;
-
     int mArtistRole = Qt::DisplayRole;
 
     int mTitleRole = Qt::DisplayRole;
@@ -292,6 +280,8 @@ private:
 
     int mDatabaseIdRole = Qt::DisplayRole;
 
+    int mTrackTypeIdRole = Qt::DisplayRole;
+
     int mAlbumIdRole = Qt::DisplayRole;
 
     int mIsValidRole = Qt::DisplayRole;
@@ -310,12 +300,12 @@ private:
 
     qulonglong mOldDatabaseId = 0;
 
+    ElisaUtils::PlayListEntryType mOldTrackType = ElisaUtils::Unknown;
+
     qulonglong mOldAlbumId = 0;
 
     bool mOldIsValid = false;
 
-    int mOldRemainingTracks = 0;
-
 };
 
 
