@@ -16,8 +16,9 @@
  */
 
 #include "mediaplaylist.h"
-#include "databaseinterface.h"
-#include "musicaudiotrack.h"
+
+#include "playListLogging.h"
+#include "datatypes.h"
 #include "musiclistenersmanager.h"
 
 #include <QUrl>
@@ -28,6 +29,7 @@
 #include <QJsonArray>
 #include <QJsonDocument>
 #include <QDebug>
+#include <QRandomGenerator>
 
 #include <algorithm>
 
@@ -37,12 +39,16 @@ public:
 
     QList<MediaPlayListEntry> mData;
 
-    QList<DatabaseInterface::TrackDataType> mTrackData;
+    QList<DataTypes::TrackDataType> mTrackData;
 
     MusicListenersManager* mMusicListenersManager = nullptr;
 
+    QPersistentModelIndex mPreviousTrack;
+
     QPersistentModelIndex mCurrentTrack;
 
+    QPersistentModelIndex mNextTrack;
+
     QVariantMap mPersistentState;
 
     QMediaPlaylist mLoadPlaylist;
@@ -55,18 +61,14 @@ public:
 
     bool mForceUndo = false;
 
+    QList<int> mRandomPositions = {0, 0, 0};
+
 };
 
 MediaPlayList::MediaPlayList(QObject *parent) : QAbstractListModel(parent), d(new MediaPlayListPrivate), dOld(new MediaPlayListPrivate)
 {
     connect(&d->mLoadPlaylist, &QMediaPlaylist::loaded, this, &MediaPlayList::loadPlayListLoaded);
     connect(&d->mLoadPlaylist, &QMediaPlaylist::loadFailed, this, &MediaPlayList::loadPlayListLoadFailed);
-
-    auto currentMsecTime = QTime::currentTime().msec();
-
-    if (currentMsecTime != -1) {
-        seedRandomGenerator(static_cast<unsigned int>(currentMsecTime));
-    }
 }
 
 MediaPlayList::~MediaPlayList()
@@ -113,6 +115,7 @@ QHash<int, QByteArray> MediaPlayList::roleNames() const
     roles[static_cast<int>(ColumnsRoles::TrackDataRole)] = "trackData";
     roles[static_cast<int>(ColumnsRoles::AlbumIdRole)] = "albumId";
     roles[static_cast<int>(ColumnsRoles::AlbumSectionRole)] = "albumSection";
+    roles[static_cast<int>(ColumnsRoles::ElementTypeRole)] = "entryType";
 
     return roles;
 }
@@ -121,6 +124,10 @@ QVariant MediaPlayList::data(const QModelIndex &index, int role) const
 {
     auto result = QVariant();
 
+    if (!index.isValid()) {
+        return result;
+    }
+
     if (d->mData[index.row()].mIsValid) {
         switch(role)
         {
@@ -142,11 +149,29 @@ QVariant MediaPlayList::data(const QModelIndex &index, int role) const
         }
         case ColumnsRoles::AlbumSectionRole:
             result = QJsonDocument{QJsonArray{d->mTrackData[index.row()][TrackDataType::key_type::AlbumRole].toString(),
-                                              d->mTrackData[index.row()][TrackDataType::key_type::AlbumArtistRole].toString(),
-                                              d->mTrackData[index.row()][TrackDataType::key_type::ImageUrlRole].toUrl().toString()}}.toJson();
+                    d->mTrackData[index.row()][TrackDataType::key_type::AlbumArtistRole].toString(),
+                    d->mTrackData[index.row()][TrackDataType::key_type::ImageUrlRole].toUrl().toString()}}.toJson();
             break;
+        case ColumnsRoles::TitleRole:
+        {
+            const auto &trackData = d->mTrackData[index.row()];
+            auto titleData = trackData[TrackDataType::key_type::TitleRole];
+            if (titleData.toString().isEmpty()) {
+                result = trackData[TrackDataType::key_type::ResourceRole].toUrl().fileName();
+            } else {
+                result = titleData;
+            }
+            break;
+        }
         default:
-            result = d->mTrackData[index.row()][static_cast<TrackDataType::key_type>(role)];
+            const auto &trackData = d->mTrackData[index.row()];
+            auto roleEnum = static_cast<TrackDataType::key_type>(role);
+            auto itData = trackData.find(roleEnum);
+            if (itData != trackData.end()) {
+                result = itData.value();
+            } else {
+                result = {};
+            }
         }
     } else {
         switch(role)
@@ -186,8 +211,8 @@ QVariant MediaPlayList::data(const QModelIndex &index, int role) const
             break;
         case ColumnsRoles::AlbumSectionRole:
             result = QJsonDocument{QJsonArray{d->mData[index.row()].mAlbum.toString(),
-                                              d->mData[index.row()].mArtist.toString(),
-                                              QUrl(QStringLiteral("image://icon/error")).toString()}}.toJson();
+                    d->mData[index.row()].mArtist.toString(),
+                    QUrl(QStringLiteral("image://icon/error")).toString()}}.toJson();
             break;
         default:
             result = {};
@@ -230,6 +255,32 @@ bool MediaPlayList::setData(const QModelIndex &index, const QVariant &value, int
 
         break;
     }
+    case ColumnsRoles::TitleRole:
+    {
+        modelModified = true;
+        d->mData[index.row()].mTitle = value;
+        d->mTrackData[index.row()][static_cast<TrackDataType::key_type>(role)] = value;
+        Q_EMIT dataChanged(index, index, {role});
+
+        if (!d->mCurrentTrack.isValid()) {
+            resetCurrentTrack();
+        }
+
+        break;
+    }
+    case ColumnsRoles::ArtistRole:
+    {
+        modelModified = true;
+        d->mData[index.row()].mArtist = value;
+        d->mTrackData[index.row()][static_cast<TrackDataType::key_type>(role)] = value;
+        Q_EMIT dataChanged(index, index, {role});
+
+        if (!d->mCurrentTrack.isValid()) {
+            resetCurrentTrack();
+        }
+
+        break;
+    }
     default:
         modelModified = false;
     }
@@ -262,12 +313,20 @@ bool MediaPlayList::removeRows(int row, int count, const QModelIndex &parent)
             }
         }
     }
+    if (!d->mNextTrack.isValid() || !d->mPreviousTrack.isValid()) {
+        notifyPreviousAndNextTracks();
+    }
 
     if (!d->mCurrentTrack.isValid() && rowCount(parent) <= row) {
         resetCurrentTrack();
     }
 
+    if (d->mRandomPlay) {
+        createRandomList();
+    }
+
     Q_EMIT tracksCountChanged();
+    Q_EMIT remainingTracksChanged();
     Q_EMIT persistentStateChanged();
 
     return false;
@@ -324,10 +383,13 @@ void MediaPlayList::enqueueRestoredEntry(const MediaPlayListEntry &newEntry)
     }
 
     Q_EMIT tracksCountChanged();
+    Q_EMIT remainingTracksChanged();
     Q_EMIT persistentStateChanged();
 
     if (!newEntry.mIsValid) {
-        if (newEntry.mTrackUrl.isValid()) {
+        if (newEntry.mEntryType == ElisaUtils::Radio) {
+            Q_EMIT newEntryInList(newEntry.mId, {}, ElisaUtils::Radio);
+        } else if (newEntry.mTrackUrl.isValid()) {
             auto entryURL = newEntry.mTrackUrl.toUrl();
             if (entryURL.isLocalFile()) {
                 auto entryString =  entryURL.toLocalFile();
@@ -338,11 +400,11 @@ void MediaPlayList::enqueueRestoredEntry(const MediaPlayListEntry &newEntry)
                 Q_EMIT newEntryInList(0, entryString, ElisaUtils::FileName);
             }
         } else {
-            Q_EMIT newTrackByNameInList(newEntry.mTitle.toString(),
-                                        newEntry.mArtist.toString(),
-                                        newEntry.mAlbum.toString(),
-                                        newEntry.mTrackNumber.toInt(),
-                                        newEntry.mDiscNumber.toInt());
+            Q_EMIT newTrackByNameInList(newEntry.mTitle,
+                                        newEntry.mArtist,
+                                        newEntry.mAlbum,
+                                        newEntry.mTrackNumber,
+                                        newEntry.mDiscNumber);
         }
     } else {
         Q_EMIT newEntryInList(newEntry.mId, {}, ElisaUtils::Track);
@@ -359,6 +421,7 @@ void MediaPlayList::enqueueRestoredEntry(const MediaPlayListEntry &newEntry)
 
 void MediaPlayList::enqueueArtist(const QString &artistName)
 {
+    qCDebug(orgKdeElisaPlayList()) << "MediaPlayList::enqueueArtist" << artistName;
     enqueueCommon();
 
     auto newEntry = MediaPlayListEntry{artistName};
@@ -375,30 +438,54 @@ void MediaPlayList::enqueueArtist(const QString &artistName)
     }
 
     Q_EMIT tracksCountChanged();
+    Q_EMIT remainingTracksChanged();
     Q_EMIT newEntryInList(0, artistName, newEntry.mEntryType);
     Q_EMIT persistentStateChanged();
 }
 
-void MediaPlayList::enqueueFilesList(const ElisaUtils::EntryDataList &newEntries)
+void MediaPlayList::enqueueFilesList(const ElisaUtils::EntryDataList &newEntries, ElisaUtils::PlayListEntryType databaseIdType)
 {
+    qCDebug(orgKdeElisaPlayList()) << "MediaPlayList::enqueueFilesList";
     enqueueCommon();
 
     beginInsertRows(QModelIndex(), d->mData.size(), d->mData.size() + newEntries.size() - 1);
     for (const auto &oneTrackUrl : newEntries) {
-        auto newEntry = MediaPlayListEntry(QUrl::fromLocalFile(std::get<1>(oneTrackUrl)));
-        newEntry.mEntryType = ElisaUtils::FileName;
-        d->mData.push_back(newEntry);
-        d->mTrackData.push_back({});
-        if (newEntry.mTrackUrl.isValid()) {
-            auto entryURL = newEntry.mTrackUrl.toUrl();
-            if (entryURL.isLocalFile()) {
-                auto entryString =  entryURL.toLocalFile();
-                QFileInfo newTrackFile(entryString);
-                if (newTrackFile.exists()) {
+        switch (databaseIdType)
+        {
+        case ElisaUtils::Radio:
+        case ElisaUtils::Track:
+        case ElisaUtils::FileName:
+        {
+            const auto &trackUrl = std::get<2>(oneTrackUrl);
+            auto newEntry = MediaPlayListEntry(trackUrl);
+            newEntry.mEntryType = databaseIdType;
+            d->mData.push_back(newEntry);
+            if (trackUrl.isValid()) {
+                if (trackUrl.isLocalFile()) {
+                    d->mTrackData.push_back({{DataTypes::ColumnsRoles::ResourceRole, trackUrl}});
+                    auto entryString =  trackUrl.toLocalFile();
+                    QFileInfo newTrackFile(entryString);
+                    if (newTrackFile.exists()) {
+                        d->mData.last().mIsValid = true;
+                    }
+                } else {
+                    d->mTrackData.push_back({{DataTypes::ColumnsRoles::ResourceRole, trackUrl},
+                                             {DataTypes::ColumnsRoles::TitleRole, trackUrl.fileName()}});
                     d->mData.last().mIsValid = true;
                 }
-                Q_EMIT newEntryInList(0, entryString, newEntry.mEntryType);
+                Q_EMIT newUrlInList(trackUrl, newEntry.mEntryType);
+            } else {
+                d->mTrackData.push_back({});
             }
+            break;
+        }
+        case ElisaUtils::Album:
+        case ElisaUtils::Artist:
+        case ElisaUtils::Genre:
+        case ElisaUtils::Lyricist:
+        case ElisaUtils::Composer:
+        case ElisaUtils::Unknown:
+            break;
         }
     }
     endInsertRows();
@@ -409,18 +496,20 @@ void MediaPlayList::enqueueFilesList(const ElisaUtils::EntryDataList &newEntries
     }
 
     Q_EMIT tracksCountChanged();
+    Q_EMIT remainingTracksChanged();
     Q_EMIT persistentStateChanged();
 
     Q_EMIT dataChanged(index(rowCount() - 1, 0), index(rowCount() - 1, 0), {MediaPlayList::IsPlayingRole});
 }
 
-void MediaPlayList::enqueueTracksListById(const ElisaUtils::EntryDataList &newEntries)
+void MediaPlayList::enqueueTracksListById(const ElisaUtils::EntryDataList &newEntries, ElisaUtils::PlayListEntryType type)
 {
+    qCDebug(orgKdeElisaPlayList()) << "MediaPlayList::enqueueTracksListById" << newEntries.size() << type;
     enqueueCommon();
 
     beginInsertRows(QModelIndex(), d->mData.size(), d->mData.size() + newEntries.size() - 1);
     for (const auto &newTrack : newEntries) {
-        auto newMediaPlayListEntry = MediaPlayListEntry{std::get<0>(newTrack), std::get<1>(newTrack), ElisaUtils::Track};
+        auto newMediaPlayListEntry = MediaPlayListEntry{std::get<0>(newTrack), std::get<1>(newTrack), type};
         d->mData.push_back(newMediaPlayListEntry);
         d->mTrackData.push_back({});
         Q_EMIT newEntryInList(newMediaPlayListEntry.mId, newMediaPlayListEntry.mTitle.toString(), newMediaPlayListEntry.mEntryType);
@@ -433,6 +522,7 @@ void MediaPlayList::enqueueTracksListById(const ElisaUtils::EntryDataList &newEn
     }
 
     Q_EMIT tracksCountChanged();
+    Q_EMIT remainingTracksChanged();
     Q_EMIT persistentStateChanged();
 
     Q_EMIT dataChanged(index(rowCount() - 1, 0), index(rowCount() - 1, 0), {MediaPlayList::IsPlayingRole});
@@ -440,6 +530,7 @@ void MediaPlayList::enqueueTracksListById(const ElisaUtils::EntryDataList &newEn
 
 void MediaPlayList::enqueueOneEntry(const ElisaUtils::EntryData &entryData, ElisaUtils::PlayListEntryType type)
 {
+    qCDebug(orgKdeElisaPlayList()) << "MediaPlayList::enqueueOneEntry" << std::get<0>(entryData) << std::get<1>(entryData) << type;
     enqueueCommon();
 
     beginInsertRows(QModelIndex(), d->mData.size(), d->mData.size());
@@ -449,28 +540,40 @@ void MediaPlayList::enqueueOneEntry(const ElisaUtils::EntryData &entryData, Elis
     endInsertRows();
 
     Q_EMIT tracksCountChanged();
+    Q_EMIT remainingTracksChanged();
     Q_EMIT persistentStateChanged();
 }
 
 void MediaPlayList::enqueueMultipleEntries(const ElisaUtils::EntryDataList &entriesData, ElisaUtils::PlayListEntryType type)
 {
+    qCDebug(orgKdeElisaPlayList()) << "MediaPlayList::enqueueMultipleEntries" << entriesData.size() << type;
     enqueueCommon();
 
     beginInsertRows(QModelIndex(), d->mData.size(), d->mData.size() + entriesData.size() - 1);
     for (const auto &entryData : entriesData) {
-        d->mData.push_back(MediaPlayListEntry{std::get<0>(entryData), std::get<1>(entryData), type});
+        if (std::get<2>(entryData).isValid()) {
+            d->mData.push_back(MediaPlayListEntry{std::get<2>(entryData)});
+        } else {
+            d->mData.push_back(MediaPlayListEntry{std::get<0>(entryData), std::get<1>(entryData), type});
+        }
         d->mTrackData.push_back({});
-        Q_EMIT newEntryInList(std::get<0>(entryData), std::get<1>(entryData), type);
+        if (std::get<2>(entryData).isValid()) {
+            Q_EMIT newUrlInList(std::get<2>(entryData), type);
+        } else {
+            Q_EMIT newEntryInList(std::get<0>(entryData), std::get<1>(entryData), type);
+        }
     }
     endInsertRows();
 
     Q_EMIT tracksCountChanged();
+    Q_EMIT remainingTracksChanged();
     Q_EMIT persistentStateChanged();
 }
 
 void MediaPlayList::replaceAndPlay(const ElisaUtils::EntryData &newEntry,
                                    ElisaUtils::PlayListEntryType databaseIdType)
 {
+    qCDebug(orgKdeElisaPlayList()) << "MediaPlayList::replaceAndPlay" << std::get<0>(newEntry) << std::get<1>(newEntry) << databaseIdType;
     enqueue(newEntry, databaseIdType, ElisaUtils::PlayListEnqueueMode::ReplacePlayList, ElisaUtils::PlayListEnqueueTriggerPlay::TriggerPlay);
 }
 
@@ -496,6 +599,7 @@ void MediaPlayList::clearPlayList(bool prepareUndo)
 
     displayOrHideUndoInline(true);
     Q_EMIT tracksCountChanged();
+    Q_EMIT remainingTracksChanged();
     Q_EMIT persistentStateChanged();
 }
 
@@ -547,6 +651,7 @@ void MediaPlayList::undoClearPlayList()
     }
 
     Q_EMIT tracksCountChanged();
+    Q_EMIT remainingTracksChanged();
     Q_EMIT persistentStateChanged();
 
     Q_EMIT dataChanged(index(rowCount() - 1, 0), index(rowCount() - 1, 0), {MediaPlayList::IsPlayingRole});
@@ -554,6 +659,14 @@ void MediaPlayList::undoClearPlayList()
     Q_EMIT undoClearPlayListPlayer();
 }
 
+void MediaPlayList::updateRadioData(const QVariant &value, int role)
+{
+    auto convertedRole = static_cast<ColumnsRoles>(role);
+    if (d->mCurrentTrack.data(convertedRole) != value) {
+        this->setData(d->mCurrentTrack, value, role);
+    }
+}
+
 void MediaPlayList::enqueueCommon()
 {
     displayOrHideUndoInline(false);
@@ -571,6 +684,12 @@ void MediaPlayList::copyD()
     dOld->mRepeatPlay = d->mRepeatPlay;
 }
 
+void MediaPlayList::loadPlaylist(const QString &localFileName)
+{
+    d->mLoadPlaylist.clear();
+    d->mLoadPlaylist.load(QUrl::fromLocalFile(localFileName), "m3u");
+}
+
 void MediaPlayList::loadPlaylist(const QUrl &fileName)
 {
     d->mLoadPlaylist.clear();
@@ -593,7 +712,14 @@ void MediaPlayList::enqueue(qulonglong newEntryDatabaseId,
                             ElisaUtils::PlayListEnqueueMode enqueueMode,
                             ElisaUtils::PlayListEnqueueTriggerPlay triggerPlay)
 {
-    enqueue(ElisaUtils::EntryData{newEntryDatabaseId, newEntryTitle}, databaseIdType, enqueueMode, triggerPlay);
+    enqueue(ElisaUtils::EntryData{newEntryDatabaseId, newEntryTitle, {}}, databaseIdType, enqueueMode, triggerPlay);
+}
+
+void MediaPlayList::enqueue(const QUrl &entryUrl, ElisaUtils::PlayListEntryType databaseIdType,
+                            ElisaUtils::PlayListEnqueueMode enqueueMode,
+                            ElisaUtils::PlayListEnqueueTriggerPlay triggerPlay)
+{
+    enqueue(ElisaUtils::EntryData{{}, {}, entryUrl}, databaseIdType, enqueueMode, triggerPlay);
 }
 
 void MediaPlayList::enqueue(const ElisaUtils::EntryData &newEntry,
@@ -615,11 +741,18 @@ void MediaPlayList::enqueue(const ElisaUtils::EntryData &newEntry,
     case ElisaUtils::Album:
     case ElisaUtils::Artist:
     case ElisaUtils::Genre:
-    case ElisaUtils::Track:
         enqueueOneEntry(newEntry, databaseIdType);
         break;
+    case ElisaUtils::Track:
+    case ElisaUtils::Radio:
+        if (std::get<2>(newEntry).isValid()) {
+            enqueueFilesList({newEntry}, databaseIdType);
+        } else {
+            enqueueOneEntry(newEntry, databaseIdType);
+        }
+        break;
     case ElisaUtils::FileName:
-        enqueueFilesList({newEntry});
+        enqueueFilesList({newEntry}, databaseIdType);
         break;
     case ElisaUtils::Lyricist:
     case ElisaUtils::Composer:
@@ -657,10 +790,11 @@ void MediaPlayList::enqueue(const ElisaUtils::EntryDataList &newEntries,
     switch (databaseIdType)
     {
     case ElisaUtils::Track:
-        enqueueTracksListById(newEntries);
+    case ElisaUtils::Radio:
+        enqueueMultipleEntries(newEntries, databaseIdType);
         break;
     case ElisaUtils::FileName:
-        enqueueFilesList(newEntries);
+        enqueueFilesList(newEntries, databaseIdType);
         break;
     case ElisaUtils::Album:
     case ElisaUtils::Artist:
@@ -708,11 +842,24 @@ QVariantMap MediaPlayList::persistentState() const
         if (oneEntry.mIsValid) {
             const auto &oneTrack = d->mTrackData[trackIndex];
 
+            oneData.push_back(QString::number(oneTrack.databaseId()));
             oneData.push_back(oneTrack.title());
             oneData.push_back(oneTrack.artist());
-            oneData.push_back(oneTrack.album());
-            oneData.push_back(QString::number(oneTrack.trackNumber()));
-            oneData.push_back(QString::number(oneTrack.discNumber()));
+            if (oneTrack.hasAlbum()) {
+                oneData.push_back(oneTrack.album());
+            } else {
+                oneData.push_back({});
+            }
+            if (oneTrack.hasTrackNumber()) {
+                oneData.push_back(QString::number(oneTrack.trackNumber()));
+            } else {
+                oneData.push_back({});
+            }
+            if (oneTrack.hasDiscNumber()) {
+                oneData.push_back(QString::number(oneTrack.discNumber()));
+            } else {
+                oneData.push_back({});
+            }
             oneData.push_back(QString::number(oneEntry.mEntryType));
 
             result.push_back(QVariant(oneData));
@@ -737,11 +884,21 @@ int MediaPlayList::tracksCount() const
     return rowCount();
 }
 
+QPersistentModelIndex MediaPlayList::previousTrack() const
+{
+    return d->mPreviousTrack;
+}
+
 QPersistentModelIndex MediaPlayList::currentTrack() const
 {
     return d->mCurrentTrack;
 }
 
+QPersistentModelIndex MediaPlayList::nextTrack() const
+{
+    return d->mNextTrack;
+}
+
 int MediaPlayList::currentTrackRow() const
 {
     return d->mCurrentTrack.row();
@@ -771,19 +928,20 @@ void MediaPlayList::setPersistentState(const QVariantMap &persistentStateValue)
 
     for (auto &oneData : persistentState) {
         auto trackData = oneData.toStringList();
-        if (trackData.size() != 6) {
+        if (trackData.size() != 7) {
             continue;
         }
 
-        auto restoredTitle = trackData[0];
-        auto restoredArtist = trackData[1];
-        auto restoredAlbum = trackData[2];
-        auto restoredTrackNumber = trackData[3].toInt();
-        auto restoredDiscNumber = trackData[4].toInt();
+        auto restoredId = trackData[0].toULongLong();
+        auto restoredTitle = trackData[1];
+        auto restoredArtist = trackData[2];
+        auto restoredAlbum = trackData[3];
+        auto restoredTrackNumber = trackData[4];
+        auto restoredDiscNumber = trackData[5];
 
-        ElisaUtils::PlayListEntryType mEntryType = static_cast<ElisaUtils::PlayListEntryType>(trackData[5].toInt());
+        auto mEntryType = static_cast<ElisaUtils::PlayListEntryType>(trackData[6].toInt());
 
-        enqueueRestoredEntry({restoredTitle, restoredArtist, restoredAlbum, restoredTrackNumber, restoredDiscNumber, mEntryType});
+        enqueueRestoredEntry({restoredId, restoredTitle, restoredArtist, restoredAlbum, restoredTrackNumber, restoredDiscNumber, mEntryType});
     }
 
     restorePlayListPosition();
@@ -800,6 +958,9 @@ void MediaPlayList::removeSelection(QList<int> selection)
     for (auto oneItem : selection) {
         removeRow(oneItem);
     }
+    if (d->mRandomPlay) {
+        createRandomList();
+    }
 }
 
 void MediaPlayList::tracksListAdded(qulonglong newDatabaseId,
@@ -837,8 +998,8 @@ void MediaPlayList::tracksListAdded(qulonglong newDatabaseId,
             for (int trackIndex = 1; trackIndex < tracks.size(); ++trackIndex) {
                 auto newEntry = MediaPlayListEntry{tracks[trackIndex]};
                 newEntry.mEntryType = ElisaUtils::Track;
-                d->mData.push_back(newEntry);
-                d->mTrackData.push_back(tracks[trackIndex]);
+                d->mData.insert(playListIndex + trackIndex, newEntry);
+                d->mTrackData.insert(playListIndex + trackIndex, tracks[trackIndex]);
             }
             endInsertRows();
 
@@ -848,6 +1009,7 @@ void MediaPlayList::tracksListAdded(qulonglong newDatabaseId,
             }
 
             Q_EMIT tracksCountChanged();
+            Q_EMIT remainingTracksChanged();
         }
         Q_EMIT persistentStateChanged();
     }
@@ -892,20 +1054,8 @@ void MediaPlayList::trackChanged(const TrackDataType &track)
                 resetCurrentTrack();
             }
             continue;
-        } else if (oneEntry.mEntryType != ElisaUtils::Artist && !oneEntry.mIsValid && !oneEntry.mTrackUrl.isValid()) {
-            if (track.title() != oneEntry.mTitle) {
-                continue;
-            }
-
-            if (track.album() != oneEntry.mAlbum) {
-                continue;
-            }
-
-            if (track.trackNumber() != oneEntry.mTrackNumber) {
-                continue;
-            }
-
-            if (track.discNumber() != oneEntry.mDiscNumber) {
+        } else if (oneEntry.mEntryType == ElisaUtils::Radio ) {
+            if (track.databaseId() != oneEntry.mId) {
                 continue;
             }
 
@@ -921,6 +1071,44 @@ void MediaPlayList::trackChanged(const TrackDataType &track)
                 resetCurrentTrack();
             }
 
+            break;
+        } else if (oneEntry.mEntryType != ElisaUtils::Artist && !oneEntry.mIsValid && !oneEntry.mTrackUrl.isValid()) {
+            if (track.find(TrackDataType::key_type::TitleRole) != track.end() &&
+                    track.title() != oneEntry.mTitle) {
+                continue;
+            }
+
+            if (track.find(TrackDataType::key_type::AlbumRole) != track.end() &&
+                    track.album() != oneEntry.mAlbum) {
+                continue;
+            }
+
+            if (track.find(TrackDataType::key_type::TrackNumberRole) != track.end() &&
+                    track.trackNumber() != oneEntry.mTrackNumber) {
+                continue;
+            }
+
+            if (track.find(TrackDataType::key_type::DiscNumberRole) != track.end() &&
+                    track.discNumber() != oneEntry.mDiscNumber) {
+                continue;
+            }
+
+            d->mTrackData[i] = track;
+            oneEntry.mId = track.databaseId();
+            oneEntry.mIsValid = true;
+
+            Q_EMIT dataChanged(index(i, 0), index(i, 0), {});
+
+            restorePlayListPosition();
+
+            if (!d->mCurrentTrack.isValid()) {
+                resetCurrentTrack();
+            } else if (i == d->mCurrentTrack.row()) {
+                notifyCurrentTrackChanged();
+            } else if (i == d->mNextTrack.row() || i == d->mPreviousTrack.row()) {
+                notifyPreviousAndNextTracks();
+            }
+
             break;
         } else if (oneEntry.mEntryType != ElisaUtils::Artist && !oneEntry.mIsValid && oneEntry.mTrackUrl.isValid()) {
             if (track.resourceURI() != oneEntry.mTrackUrl) {
@@ -937,6 +1125,10 @@ void MediaPlayList::trackChanged(const TrackDataType &track)
 
             if (!d->mCurrentTrack.isValid()) {
                 resetCurrentTrack();
+            } else if (i == d->mCurrentTrack.row()) {
+                notifyCurrentTrackChanged();
+            } else if (i == d->mNextTrack.row() || i == d->mPreviousTrack.row()) {
+                notifyPreviousAndNextTracks();
             }
 
             break;
@@ -985,14 +1177,23 @@ void MediaPlayList::setMusicListenersManager(MusicListenersManager *musicListene
 
 void MediaPlayList::setRandomPlay(bool value)
 {
-    d->mRandomPlay = value;
-    Q_EMIT randomPlayChanged();
+    if (d->mRandomPlay != value) {
+        d->mRandomPlay = value;
+        createRandomList();
+        Q_EMIT randomPlayChanged();
+        Q_EMIT remainingTracksChanged();
+        notifyPreviousAndNextTracks();
+    }
 }
 
 void MediaPlayList::setRepeatPlay(bool value)
 {
-    d->mRepeatPlay = value;
-    Q_EMIT repeatPlayChanged();
+    if (d->mRepeatPlay != value) {
+        d->mRepeatPlay = value;
+        Q_EMIT repeatPlayChanged();
+        Q_EMIT remainingTracksChanged();
+        notifyPreviousAndNextTracks();
+    }
 }
 
 void MediaPlayList::displayOrHideUndoInline(bool value)
@@ -1012,27 +1213,21 @@ void MediaPlayList::skipNextTrack()
         return;
     }
 
-    if (!d->mRandomPlay && (d->mCurrentTrack.row() >= (rowCount() - 1))) {
-        if (!d->mRepeatPlay) {
-            Q_EMIT playListFinished();
-        }
-
-        if (rowCount() == 1) {
-            d->mCurrentTrack = QPersistentModelIndex{};
-            notifyCurrentTrackChanged();
-        }
-
-        resetCurrentTrack();
-
-        return;
-    }
-
     if (d->mRandomPlay) {
-        int randomValue = qrand();
-        randomValue = randomValue % (rowCount());
-        d->mCurrentTrack = index(randomValue, 0);
+        d->mRandomPositions.removeFirst();
+        d->mCurrentTrack = index(d->mRandomPositions.at(1), 0);
+        if (rowCount()) {
+            d->mRandomPositions.append(QRandomGenerator::global()->bounded(rowCount()));
+        }
     } else {
-        d->mCurrentTrack = index(d->mCurrentTrack.row() + 1, 0);
+        if (d->mCurrentTrack.row() >= rowCount() - 1) {
+            d->mCurrentTrack = index(0, 0);
+            if (!d->mRepeatPlay) {
+                Q_EMIT playListFinished();
+            }
+        } else {
+            d->mCurrentTrack = index(d->mCurrentTrack.row() + 1, 0);
+        }
     }
 
     notifyCurrentTrackChanged();
@@ -1044,31 +1239,25 @@ void MediaPlayList::skipPreviousTrack()
         return;
     }
 
-    if (!d->mRandomPlay && !d->mRepeatPlay && d->mCurrentTrack.row() <= 0) {
-        return;
-    }
-
     if (d->mRandomPlay) {
-        int randomValue = qrand();
-        randomValue = randomValue % (rowCount());
-        d->mCurrentTrack = index(randomValue, 0);
+        d->mRandomPositions.removeLast();
+        d->mCurrentTrack = index(d->mRandomPositions.at(0), 0);
+        if (rowCount()) {
+            d->mRandomPositions.prepend(QRandomGenerator::global()->bounded(rowCount()));
+        }
     } else {
-        if (d->mRepeatPlay) {
-            if (d->mCurrentTrack.row() == 0) {
+        if (d->mCurrentTrack.row() == 0) {
+            if (d->mRepeatPlay) {
                 d->mCurrentTrack = index(rowCount() - 1, 0);
             } else {
-                d->mCurrentTrack = index(d->mCurrentTrack.row() - 1, 0);
+                return;
             }
         } else {
-            d->mCurrentTrack = index(d->mCurrentTrack.row() - 1, d->mCurrentTrack.column(), d->mCurrentTrack.parent());
+            d->mCurrentTrack = index(d->mCurrentTrack.row() - 1, 0);
         }
     }
-    notifyCurrentTrackChanged();
-}
 
-void MediaPlayList::seedRandomGenerator(uint seed)
-{
-    qsrand(seed);
+    notifyCurrentTrackChanged();
 }
 
 void MediaPlayList::switchTo(int row)
@@ -1077,7 +1266,12 @@ void MediaPlayList::switchTo(int row)
         return;
     }
 
-    d->mCurrentTrack = index(row, 0);
+    if (d->mRandomPlay) {
+        d->mCurrentTrack = index(row, 0);
+        d->mRandomPositions.replace(1, row);
+    } else {
+        d->mCurrentTrack = index(row, 0);
+    }
 
     notifyCurrentTrackChanged();
 }
@@ -1104,7 +1298,7 @@ void MediaPlayList::loadPlayListLoaded()
     clearPlayList();
 
     for (int i = 0; i < d->mLoadPlaylist.mediaCount(); ++i) {
-        enqueue(ElisaUtils::EntryData{0, d->mLoadPlaylist.media(i).canonicalUrl().toLocalFile()}, ElisaUtils::FileName);
+        enqueue(ElisaUtils::EntryData{{}, {}, d->mLoadPlaylist.media(i).canonicalUrl()}, ElisaUtils::FileName);
     }
 
     restorePlayListPosition();
@@ -1136,10 +1330,59 @@ void MediaPlayList::resetCurrentTrack()
     }
 }
 
+void MediaPlayList::notifyPreviousAndNextTracks()
+{
+    if (!d->mCurrentTrack.isValid()) {
+        d->mPreviousTrack = QPersistentModelIndex();
+        d->mNextTrack = QPersistentModelIndex();
+    }
+    auto mOldPreviousTrack = d->mPreviousTrack;
+    auto mOldNextTrack = d->mNextTrack;
+    // use random list for previous and next types
+    if (d->mRandomPlay) {
+        d->mPreviousTrack = index(d->mRandomPositions.first(), 0);
+        d->mNextTrack = index(d->mRandomPositions.last(), 0);
+    } else if (d->mRepeatPlay) {
+        // forward to end or begin when repeating
+        if (d->mCurrentTrack.row() == 0) {
+            d->mPreviousTrack = index(rowCount() - 1, 0);
+        } else {
+            d->mPreviousTrack = index(d->mCurrentTrack.row() - 1, 0);
+        }
+        if (d->mCurrentTrack.row() == rowCount() - 1) {
+            d->mNextTrack = index(0, 0);
+        } else {
+            d->mNextTrack = index(d->mCurrentTrack.row() + 1, 0);
+        }
+    } else {
+        // return nothing if no tracks available
+        if (d->mCurrentTrack.row() == 0) {
+            d->mPreviousTrack = QPersistentModelIndex();
+        } else {
+            d->mPreviousTrack = index(d->mCurrentTrack.row() - 1, 0);
+        }
+        if (d->mCurrentTrack.row() == rowCount() - 1) {
+            d->mNextTrack = QPersistentModelIndex();
+        } else {
+            d->mNextTrack = index(d->mCurrentTrack.row() + 1, 0);
+        }
+    }
+    if (d->mPreviousTrack != mOldPreviousTrack) {
+        Q_EMIT previousTrackChanged(d->mPreviousTrack);
+    }
+    if (d->mNextTrack != mOldNextTrack) {
+        Q_EMIT nextTrackChanged(d->mNextTrack);
+    }
+}
+
 void MediaPlayList::notifyCurrentTrackChanged()
 {
+    // determine previous and next tracks first
+    notifyPreviousAndNextTracks();
+
     Q_EMIT currentTrackChanged(d->mCurrentTrack);
     Q_EMIT currentTrackRowChanged();
+    Q_EMIT remainingTracksChanged();
     bool currentTrackIsValid = d->mCurrentTrack.isValid();
     if (currentTrackIsValid) {
         d->mCurrentPlayListPosition = d->mCurrentTrack.row();
@@ -1167,7 +1410,6 @@ void MediaPlayList::restoreRandomPlay()
     auto randomPlayStoredValue = d->mPersistentState.find(QStringLiteral("randomPlay"));
     if (randomPlayStoredValue != d->mPersistentState.end()) {
         setRandomPlay(randomPlayStoredValue->toBool());
-        d->mPersistentState.erase(randomPlayStoredValue);
     }
 }
 
@@ -1176,7 +1418,6 @@ void MediaPlayList::restoreRepeatPlay()
     auto repeatPlayStoredValue = d->mPersistentState.find(QStringLiteral("repeatPlay"));
     if (repeatPlayStoredValue != d->mPersistentState.end()) {
         setRepeatPlay(repeatPlayStoredValue->toBool());
-        d->mPersistentState.erase(repeatPlayStoredValue);
     }
 }
 
@@ -1186,4 +1427,29 @@ QDebug operator<<(const QDebug &stream, const MediaPlayListEntry &data)
     return stream;
 }
 
+int MediaPlayList::remainingTracks() const
+{
+    if (!d->mCurrentTrack.isValid()) {
+        return -1;
+    }
+
+    if (d->mRandomPlay || d->mRepeatPlay) {
+        return -1;
+    } else {
+        return rowCount() - d->mCurrentTrack.row() - 1;
+    }
+}
+
+void MediaPlayList::createRandomList()
+{
+    if (rowCount()) {
+        for (auto& position : d->mRandomPositions) {
+            position = QRandomGenerator::global()->bounded(rowCount());
+        }
+    }
+    if (d->mCurrentTrack.isValid()) {
+        d->mRandomPositions.replace(1, d->mCurrentTrack.row());
+    }
+}
+
 #include "moc_mediaplaylist.cpp"
