@@ -20,9 +20,8 @@
 
 #include "elisaLib_export.h"
 
-#include "musicaudiotrack.h"
 #include "elisautils.h"
-#include "databaseinterface.h"
+#include "datatypes.h"
 
 #include <QAbstractListModel>
 #include <QVector>
@@ -30,6 +29,7 @@
 
 #include <utility>
 #include <tuple>
+#include <memory>
 
 class MediaPlayListPrivate;
 class MusicListenersManager;
@@ -54,10 +54,18 @@ class ELISALIB_EXPORT MediaPlayList : public QAbstractListModel
                READ tracksCount
                NOTIFY tracksCountChanged)
 
+    Q_PROPERTY(QPersistentModelIndex previousTrack
+               READ previousTrack
+               NOTIFY previousTrackChanged)
+
     Q_PROPERTY(QPersistentModelIndex currentTrack
                READ currentTrack
                NOTIFY currentTrackChanged)
 
+    Q_PROPERTY(QPersistentModelIndex nextTrack
+               READ nextTrack
+               NOTIFY nextTrackChanged)
+
     Q_PROPERTY(int currentTrackRow
                READ currentTrackRow
                NOTIFY currentTrackRowChanged)
@@ -72,22 +80,26 @@ class ELISALIB_EXPORT MediaPlayList : public QAbstractListModel
                WRITE setRepeatPlay
                NOTIFY repeatPlayChanged)
 
+    Q_PROPERTY(int remainingTracks
+               READ remainingTracks
+               NOTIFY remainingTracksChanged)
+
 public:
 
     enum ColumnsRoles {
-        TitleRole = DatabaseInterface::TitleRole,
+        TitleRole = DataTypes::TitleRole,
         SecondaryTextRole,
         ImageUrlRole,
         ShadowForImageRole,
         ChildModelRole,
         DurationRole,
         StringDurationRole,
-        MilliSecondsDurationRole,
         ArtistRole,
         AllArtistsRole,
         HighestTrackRating,
         AlbumRole,
         AlbumArtistRole,
+        IsValidAlbumArtistRole,
         TrackNumberRole,
         DiscNumberRole,
         RatingRole,
@@ -101,6 +113,7 @@ public:
         SampleRateRole,
         ResourceRole,
         IdRole,
+        ParentIdRole,
         DatabaseIdRole,
         IsSingleDiscAlbumRole,
         ContainerDataRole,
@@ -113,6 +126,7 @@ public:
         PlayCounter,
         PlayFrequency,
         ElementTypeRole,
+        LyricsRole,
         IsValidRole,
         TrackDataRole,
         CountRole,
@@ -130,9 +144,9 @@ public:
 
     Q_ENUM(PlayState)
 
-    using ListTrackDataType = DatabaseInterface::ListTrackDataType;
+    using ListTrackDataType = DataTypes::ListTrackDataType;
 
-    using TrackDataType = DatabaseInterface::TrackDataType;
+    using TrackDataType = DataTypes::TrackDataType;
 
     explicit MediaPlayList(QObject *parent = nullptr);
 
@@ -162,14 +176,20 @@ public:
 
     int tracksCount() const;
 
+    QPersistentModelIndex previousTrack() const;
+
     QPersistentModelIndex currentTrack() const;
 
+    QPersistentModelIndex nextTrack() const;
+
     int currentTrackRow() const;
 
     bool randomPlay() const;
 
     bool repeatPlay() const;
 
+    int remainingTracks() const;
+
 Q_SIGNALS:
     void displayUndoInline();
 
@@ -181,13 +201,20 @@ Q_SIGNALS:
                         const QString &entryTitle,
                         ElisaUtils::PlayListEntryType databaseIdType);
 
+    void newUrlInList(const QUrl &entryUrl,
+                      ElisaUtils::PlayListEntryType databaseIdType);
+
     void persistentStateChanged();
 
     void musicListenersManagerChanged();
 
     void tracksCountChanged();
 
-    void currentTrackChanged(QPersistentModelIndex currentTrack);
+    void previousTrackChanged(const QPersistentModelIndex &previousTrack);
+
+    void currentTrackChanged(const QPersistentModelIndex &currentTrack);
+
+    void nextTrackChanged(const QPersistentModelIndex &nextTrack);
 
     void clearPlayListPlayer();
 
@@ -207,6 +234,8 @@ Q_SIGNALS:
 
     void ensurePlay();
 
+    void remainingTracksChanged();
+
 public Q_SLOTS:
 
     void setPersistentState(const QVariantMap &persistentState);
@@ -232,10 +261,10 @@ public Q_SLOTS:
 
     void skipPreviousTrack();
 
-    void seedRandomGenerator(uint seed);
-
     void switchTo(int row);
 
+    void loadPlaylist(const QString &localFileName);
+
     void loadPlaylist(const QUrl &fileName);
 
     void enqueue(const ElisaUtils::EntryData &newEntry, ElisaUtils::PlayListEntryType databaseIdType);
@@ -247,6 +276,11 @@ public Q_SLOTS:
                  ElisaUtils::PlayListEnqueueMode enqueueMode,
                  ElisaUtils::PlayListEnqueueTriggerPlay triggerPlay);
 
+    void enqueue(const QUrl &entryUrl,
+                 ElisaUtils::PlayListEntryType databaseIdType,
+                 ElisaUtils::PlayListEnqueueMode enqueueMode,
+                 ElisaUtils::PlayListEnqueueTriggerPlay triggerPlay);
+
     void enqueue(const ElisaUtils::EntryData &newEntry,
                  ElisaUtils::PlayListEntryType databaseIdType,
                  ElisaUtils::PlayListEnqueueMode enqueueMode,
@@ -265,6 +299,8 @@ public Q_SLOTS:
 
     void undoClearPlayList();
 
+    void updateRadioData(const QVariant &value, int role);
+
 private Q_SLOTS:
 
     void loadPlayListLoaded();
@@ -284,13 +320,17 @@ private:
 
     void restoreRandomPlay();
 
+    void createRandomList();
+
     void restoreRepeatPlay();
 
+    void notifyPreviousAndNextTracks();
+
     void enqueueArtist(const QString &artistName);
 
-    void enqueueFilesList(const ElisaUtils::EntryDataList &newEntries);
+    void enqueueFilesList(const ElisaUtils::EntryDataList &newEntries, ElisaUtils::PlayListEntryType databaseIdType);
 
-    void enqueueTracksListById(const ElisaUtils::EntryDataList &newEntries);
+    void enqueueTracksListById(const ElisaUtils::EntryDataList &newEntries, ElisaUtils::PlayListEntryType type);
 
     void enqueueOneEntry(const ElisaUtils::EntryData &entryData, ElisaUtils::PlayListEntryType type);
 
@@ -319,23 +359,17 @@ public:
           mTrackNumber(trackNumber), mDiscNumber(discNumber), mEntryType(entryType) {
     }
 
-    MediaPlayListEntry(QVariant title, QVariant artist, QVariant album, QVariant trackNumber,
-                       QVariant discNumber, ElisaUtils::PlayListEntryType entryType = ElisaUtils::Unknown)
+    MediaPlayListEntry(qulonglong id, QVariant title, QVariant artist, QVariant album, QVariant trackNumber, QVariant discNumber, ElisaUtils::PlayListEntryType entryType = ElisaUtils::Unknown)
         : mTitle(std::move(title)), mAlbum(std::move(album)), mArtist(std::move(artist)),
-          mTrackNumber(std::move(trackNumber)), mDiscNumber(std::move(discNumber)), mEntryType(entryType) {
-    }
-
-    explicit MediaPlayListEntry(const MusicAudioTrack &track)
-        : mTitle(track.title()), mAlbum(track.albumName()), mTrackNumber(track.trackNumber()),
-          mDiscNumber(track.discNumber()), mId(track.databaseId()), mIsValid(true) {
+          mTrackNumber(trackNumber), mDiscNumber(discNumber), mId(id), mEntryType(entryType) {
     }
 
     explicit MediaPlayListEntry(const MediaPlayList::TrackDataType &track)
-        : mTitle(track[DatabaseInterface::TitleRole]),
-          mAlbum(track[DatabaseInterface::AlbumRole]),
-          mTrackNumber(track[DatabaseInterface::TrackNumberRole]),
-          mDiscNumber(track[DatabaseInterface::DiscNumberRole]),
-          mId(track[DatabaseInterface::DatabaseIdRole].toULongLong()),
+        : mTitle(track[DataTypes::TitleRole]),
+          mAlbum(track[DataTypes::AlbumRole]),
+          mTrackNumber(track[DataTypes::TrackNumberRole]),
+          mDiscNumber(track[DataTypes::DiscNumberRole]),
+          mId(track[DataTypes::DatabaseIdRole].toULongLong()),
           mIsValid(true) {
     }
 
