@@ -19,21 +19,15 @@
 
 #include "filescanner.h"
 
-#include <QMimeDatabase>
-
 class ModelDataLoaderPrivate
 {
 public:
 
     DatabaseInterface *mDatabase = nullptr;
 
-    QMimeDatabase mMimeDatabase;
-
-    FileScanner mFileScanner;
-
     ElisaUtils::PlayListEntryType mModelType = ElisaUtils::Unknown;
 
-    ModelDataLoader::FilterType mFilterType = ModelDataLoader::FilterType::Unknown;
+    ModelDataLoader::FilterType mFilterType = ModelDataLoader::FilterType::UnknownFilter;
 
     QString mArtist;
 
@@ -45,6 +39,8 @@ public:
 
     qulonglong mDatabaseId = 0;
 
+    FileScanner mFileScanner;
+
 };
 
 ModelDataLoader::ModelDataLoader(QObject *parent) : QObject(parent), d(std::make_unique<ModelDataLoaderPrivate>())
@@ -58,23 +54,33 @@ void ModelDataLoader::setDatabase(DatabaseInterface *database)
     d->mDatabase = database;
 
     connect(database, &DatabaseInterface::genresAdded,
-            this, &ModelDataLoader::databaseGenresAdded);
+            this, &ModelDataLoader::genresAdded);
     connect(database, &DatabaseInterface::albumsAdded,
             this, &ModelDataLoader::databaseAlbumsAdded);
     connect(database, &DatabaseInterface::albumModified,
-            this, &ModelDataLoader::databaseAlbumModified);
+            this, &ModelDataLoader::albumModified);
     connect(database, &DatabaseInterface::albumRemoved,
-            this, &ModelDataLoader::databaseAlbumRemoved);
+            this, &ModelDataLoader::albumRemoved);
     connect(database, &DatabaseInterface::tracksAdded,
             this, &ModelDataLoader::databaseTracksAdded);
     connect(database, &DatabaseInterface::trackModified,
-            this, &ModelDataLoader::databaseTrackModified);
+            this, &ModelDataLoader::trackModified);
     connect(database, &DatabaseInterface::trackRemoved,
-            this, &ModelDataLoader::databaseTrackRemoved);
+            this, &ModelDataLoader::trackRemoved);
     connect(database, &DatabaseInterface::artistsAdded,
             this, &ModelDataLoader::databaseArtistsAdded);
     connect(database, &DatabaseInterface::artistRemoved,
-            this, &ModelDataLoader::databaseArtistRemoved);
+            this, &ModelDataLoader::artistRemoved);
+    connect(this, &ModelDataLoader::saveRadioModified,
+            database, &DatabaseInterface::insertRadio);
+    connect(this, &ModelDataLoader::removeRadio,
+            database, &DatabaseInterface::removeRadio);
+    connect(database, &DatabaseInterface::radioAdded,
+            this, &ModelDataLoader::radioAdded);
+    connect(database, &DatabaseInterface::radioModified,
+            this, &ModelDataLoader::radioModified);
+    connect(database, &DatabaseInterface::radioRemoved,
+            this, &ModelDataLoader::radioRemoved);
 }
 
 void ModelDataLoader::loadData(ElisaUtils::PlayListEntryType dataType)
@@ -106,6 +112,9 @@ void ModelDataLoader::loadData(ElisaUtils::PlayListEntryType dataType)
     case ElisaUtils::FileName:
     case ElisaUtils::Unknown:
         break;
+    case ElisaUtils::Radio:
+        Q_EMIT allRadiosData(d->mDatabase->allRadiosData());
+        break;
     }
 }
 
@@ -135,6 +144,7 @@ void ModelDataLoader::loadDataByAlbumId(ElisaUtils::PlayListEntryType dataType,
         break;
     case ElisaUtils::FileName:
     case ElisaUtils::Unknown:
+    case ElisaUtils::Radio:
         break;
     }
 }
@@ -160,6 +170,7 @@ void ModelDataLoader::loadDataByGenre(ElisaUtils::PlayListEntryType dataType, co
     case ElisaUtils::Lyricist:
     case ElisaUtils::FileName:
     case ElisaUtils::Unknown:
+    case ElisaUtils::Radio:
         break;
     }
 }
@@ -185,6 +196,7 @@ void ModelDataLoader::loadDataByArtist(ElisaUtils::PlayListEntryType dataType, c
     case ElisaUtils::Lyricist:
     case ElisaUtils::FileName:
     case ElisaUtils::Unknown:
+    case ElisaUtils::Radio:
         break;
     }
 }
@@ -211,11 +223,13 @@ void ModelDataLoader::loadDataByGenreAndArtist(ElisaUtils::PlayListEntryType dat
     case ElisaUtils::Track:
     case ElisaUtils::FileName:
     case ElisaUtils::Unknown:
+    case ElisaUtils::Radio:
         break;
     }
 }
 
-void ModelDataLoader::loadDataByDatabaseId(ElisaUtils::PlayListEntryType dataType, qulonglong databaseId)
+void ModelDataLoader::loadDataByDatabaseIdAndUrl(ElisaUtils::PlayListEntryType dataType,
+                                                 qulonglong databaseId, const QUrl &url)
 {
     if (!d->mDatabase) {
         return;
@@ -227,7 +241,10 @@ void ModelDataLoader::loadDataByDatabaseId(ElisaUtils::PlayListEntryType dataTyp
     switch (dataType)
     {
     case ElisaUtils::Track:
-        Q_EMIT allTrackData(d->mDatabase->trackDataFromDatabaseId(databaseId));
+        Q_EMIT allTrackData(d->mDatabase->trackDataFromDatabaseIdAndUrl(databaseId, url));
+        break;
+    case ElisaUtils::Radio:
+        Q_EMIT allRadioData(d->mDatabase->radioDataFromDatabaseId(databaseId));
         break;
     case ElisaUtils::Album:
     case ElisaUtils::Artist:
@@ -240,20 +257,25 @@ void ModelDataLoader::loadDataByDatabaseId(ElisaUtils::PlayListEntryType dataTyp
     }
 }
 
-void ModelDataLoader::loadDataByFileName(ElisaUtils::PlayListEntryType dataType, const QUrl &fileName)
+void ModelDataLoader::loadDataByUrl(ElisaUtils::PlayListEntryType dataType, const QUrl &url)
 {
     if (!d->mDatabase) {
         return;
     }
 
-    d->mFilterType = ModelDataLoader::FilterType::Unknown;
+    d->mFilterType = ModelDataLoader::FilterType::UnknownFilter;
 
     switch (dataType)
     {
     case ElisaUtils::FileName:
     {
-        auto result = d->mFileScanner.scanOneFile(fileName, d->mMimeDatabase);
-        Q_EMIT allTrackData(result.toTrackData());
+        auto databaseId = d->mDatabase->trackIdFromFileName(url);
+        if (databaseId != 0) {
+            Q_EMIT allTrackData(d->mDatabase->trackDataFromDatabaseIdAndUrl(databaseId, url));
+        } else {
+            auto result = d->mFileScanner.scanOneFile(url);
+            Q_EMIT allTrackData(result);
+        }
         break;
     }
     case ElisaUtils::Track:
@@ -263,6 +285,7 @@ void ModelDataLoader::loadDataByFileName(ElisaUtils::PlayListEntryType dataType,
     case ElisaUtils::Genre:
     case ElisaUtils::Lyricist:
     case ElisaUtils::Unknown:
+    case ElisaUtils::Radio:
         break;
     }
 }
@@ -273,7 +296,7 @@ void ModelDataLoader::loadRecentlyPlayedData(ElisaUtils::PlayListEntryType dataT
         return;
     }
 
-    d->mFilterType = ModelDataLoader::FilterType::RecentlyPlayed;
+    d->mFilterType = ModelDataLoader::FilterType::FilterByRecentlyPlayed;
 
     switch (dataType)
     {
@@ -287,6 +310,7 @@ void ModelDataLoader::loadRecentlyPlayedData(ElisaUtils::PlayListEntryType dataT
     case ElisaUtils::Lyricist:
     case ElisaUtils::FileName:
     case ElisaUtils::Unknown:
+    case ElisaUtils::Radio:
         break;
     }
 }
@@ -297,7 +321,7 @@ void ModelDataLoader::loadFrequentlyPlayedData(ElisaUtils::PlayListEntryType dat
         return;
     }
 
-    d->mFilterType = ModelDataLoader::FilterType::FrequentlyPlayed;
+    d->mFilterType = ModelDataLoader::FilterType::FilterByFrequentlyPlayed;
 
     switch (dataType)
     {
@@ -311,6 +335,7 @@ void ModelDataLoader::loadFrequentlyPlayedData(ElisaUtils::PlayListEntryType dat
     case ElisaUtils::Lyricist:
     case ElisaUtils::FileName:
     case ElisaUtils::Unknown:
+    case ElisaUtils::Radio:
         break;
     }
 }
@@ -335,28 +360,13 @@ void ModelDataLoader::databaseTracksAdded(const ListTrackDataType &newData)
     case ModelDataLoader::FilterType::FilterByGenre:
     case ModelDataLoader::FilterType::FilterByGenreAndArtist:
     case ModelDataLoader::FilterType::FilterByArtist:
-    case ModelDataLoader::FilterType::RecentlyPlayed:
-    case ModelDataLoader::FilterType::FrequentlyPlayed:
-    case ModelDataLoader::FilterType::Unknown:
+    case ModelDataLoader::FilterType::FilterByRecentlyPlayed:
+    case ModelDataLoader::FilterType::FilterByFrequentlyPlayed:
+    case ModelDataLoader::FilterType::UnknownFilter:
         break;
     }
 }
 
-void ModelDataLoader::databaseTrackModified(const TrackDataType &modifiedTrack)
-{
-    Q_EMIT trackModified(modifiedTrack);
-}
-
-void ModelDataLoader::databaseTrackRemoved(qulonglong removedTrackId)
-{
-    Q_EMIT trackRemoved(removedTrackId);
-}
-
-void ModelDataLoader::databaseGenresAdded(const ListGenreDataType &newData)
-{
-    Q_EMIT genresAdded(newData);
-}
-
 void ModelDataLoader::databaseArtistsAdded(const ListArtistDataType &newData)
 {
     switch(d->mFilterType) {
@@ -377,18 +387,13 @@ void ModelDataLoader::databaseArtistsAdded(const ListArtistDataType &newData)
     case ModelDataLoader::FilterType::FilterByGenreAndArtist:
     case ModelDataLoader::FilterType::FilterByArtist:
     case ModelDataLoader::FilterType::FilterById:
-    case ModelDataLoader::FilterType::RecentlyPlayed:
-    case ModelDataLoader::FilterType::FrequentlyPlayed:
-    case ModelDataLoader::FilterType::Unknown:
+    case ModelDataLoader::FilterType::FilterByRecentlyPlayed:
+    case ModelDataLoader::FilterType::FilterByFrequentlyPlayed:
+    case ModelDataLoader::FilterType::UnknownFilter:
         break;
     }
 }
 
-void ModelDataLoader::databaseArtistRemoved(qulonglong removedDatabaseId)
-{
-    Q_EMIT artistRemoved(removedDatabaseId);
-}
-
 void ModelDataLoader::databaseAlbumsAdded(const ListAlbumDataType &newData)
 {
     switch(d->mFilterType) {
@@ -422,22 +427,11 @@ void ModelDataLoader::databaseAlbumsAdded(const ListAlbumDataType &newData)
     }
     case ModelDataLoader::FilterType::FilterByGenre:
     case ModelDataLoader::FilterType::FilterById:
-    case ModelDataLoader::FilterType::RecentlyPlayed:
-    case ModelDataLoader::FilterType::FrequentlyPlayed:
-    case ModelDataLoader::FilterType::Unknown:
+    case ModelDataLoader::FilterType::FilterByRecentlyPlayed:
+    case ModelDataLoader::FilterType::FilterByFrequentlyPlayed:
+    case ModelDataLoader::FilterType::UnknownFilter:
         break;
     }
 }
 
-void ModelDataLoader::databaseAlbumRemoved(qulonglong removedDatabaseId)
-{
-    Q_EMIT albumRemoved(removedDatabaseId);
-}
-
-void ModelDataLoader::databaseAlbumModified(const AlbumDataType &modifiedAlbum)
-{
-    Q_EMIT albumModified(modifiedAlbum);
-}
-
-
 #include "moc_modeldataloader.cpp"
