@@ -34,17 +34,19 @@ public:
 
     DataModel::ListTrackDataType mAllTrackData;
 
+    DataModel::ListRadioDataType mAllRadiosData;
+
     DataModel::ListAlbumDataType mAllAlbumData;
 
     DataModel::ListArtistDataType mAllArtistData;
 
     DataModel::ListGenreDataType mAllGenreData;
 
-    ModelDataLoader mDataLoader;
+    ModelDataLoader *mDataLoader = nullptr;
 
     ElisaUtils::PlayListEntryType mModelType = ElisaUtils::Unknown;
 
-    DataModel::FilterType mFilterType = DataModel::Unknown;
+    ElisaUtils::FilterType mFilterType = ElisaUtils::UnknownFilter;
 
     QString mArtist;
 
@@ -62,6 +64,8 @@ public:
 
 DataModel::DataModel(QObject *parent) : QAbstractListModel(parent), d(std::make_unique<DataModelPrivate>())
 {
+    d->mDataLoader = new ModelDataLoader;
+    connect(this, &DataModel::destroyed, d->mDataLoader, &ModelDataLoader::deleteLater);
 }
 
 DataModel::~DataModel()
@@ -84,24 +88,25 @@ QHash<int, QByteArray> DataModel::roleNames() const
 {
     auto roles = QAbstractListModel::roleNames();
 
-    roles[static_cast<int>(DatabaseInterface::ColumnsRoles::TitleRole)] = "title";
-    roles[static_cast<int>(DatabaseInterface::ColumnsRoles::SecondaryTextRole)] = "secondaryText";
-    roles[static_cast<int>(DatabaseInterface::ColumnsRoles::ImageUrlRole)] = "imageUrl";
-    roles[static_cast<int>(DatabaseInterface::ColumnsRoles::DatabaseIdRole)] = "databaseId";
-    roles[static_cast<int>(DatabaseInterface::ColumnsRoles::ElementTypeRole)] = "dataType";
+    roles[static_cast<int>(DataTypes::ColumnsRoles::TitleRole)] = "title";
+    roles[static_cast<int>(DataTypes::ColumnsRoles::SecondaryTextRole)] = "secondaryText";
+    roles[static_cast<int>(DataTypes::ColumnsRoles::ImageUrlRole)] = "imageUrl";
+    roles[static_cast<int>(DataTypes::ColumnsRoles::DatabaseIdRole)] = "databaseId";
+    roles[static_cast<int>(DataTypes::ColumnsRoles::ElementTypeRole)] = "dataType";
+    roles[static_cast<int>(DataTypes::ColumnsRoles::ResourceRole)] = "url";
 
-    roles[static_cast<int>(DatabaseInterface::ColumnsRoles::ArtistRole)] = "artist";
-    roles[static_cast<int>(DatabaseInterface::ColumnsRoles::AllArtistsRole)] = "allArtists";
-    roles[static_cast<int>(DatabaseInterface::ColumnsRoles::HighestTrackRating)] = "highestTrackRating";
-    roles[static_cast<int>(DatabaseInterface::ColumnsRoles::GenreRole)] = "genre";
+    roles[static_cast<int>(DataTypes::ColumnsRoles::ArtistRole)] = "artist";
+    roles[static_cast<int>(DataTypes::ColumnsRoles::AllArtistsRole)] = "allArtists";
+    roles[static_cast<int>(DataTypes::ColumnsRoles::HighestTrackRating)] = "highestTrackRating";
+    roles[static_cast<int>(DataTypes::ColumnsRoles::GenreRole)] = "genre";
 
-    roles[static_cast<int>(DatabaseInterface::ColumnsRoles::AlbumRole)] = "album";
-    roles[static_cast<int>(DatabaseInterface::ColumnsRoles::AlbumArtistRole)] = "albumArtist";
-    roles[static_cast<int>(DatabaseInterface::ColumnsRoles::DurationRole)] = "duration";
-    roles[static_cast<int>(DatabaseInterface::ColumnsRoles::TrackNumberRole)] = "trackNumber";
-    roles[static_cast<int>(DatabaseInterface::ColumnsRoles::DiscNumberRole)] = "discNumber";
-    roles[static_cast<int>(DatabaseInterface::ColumnsRoles::RatingRole)] = "rating";
-    roles[static_cast<int>(DatabaseInterface::ColumnsRoles::IsSingleDiscAlbumRole)] = "isSingleDiscAlbum";
+    roles[static_cast<int>(DataTypes::ColumnsRoles::AlbumRole)] = "album";
+    roles[static_cast<int>(DataTypes::ColumnsRoles::AlbumArtistRole)] = "albumArtist";
+    roles[static_cast<int>(DataTypes::ColumnsRoles::DurationRole)] = "duration";
+    roles[static_cast<int>(DataTypes::ColumnsRoles::TrackNumberRole)] = "trackNumber";
+    roles[static_cast<int>(DataTypes::ColumnsRoles::DiscNumberRole)] = "discNumber";
+    roles[static_cast<int>(DataTypes::ColumnsRoles::RatingRole)] = "rating";
+    roles[static_cast<int>(DataTypes::ColumnsRoles::IsSingleDiscAlbumRole)] = "isSingleDiscAlbum";
 
     return roles;
 }
@@ -119,14 +124,18 @@ QVariant DataModel::data(const QModelIndex &index, int role) const
 {
     auto result = QVariant();
 
-    const auto dataCount = d->mAllTrackData.size() + d->mAllAlbumData.size() + d->mAllArtistData.size() + d->mAllGenreData.size();
+    if (!index.isValid()) {
+        return result;
+    }
+
+    const auto dataCount = d->mModelType == ElisaUtils::Radio ? d->mAllRadiosData.size() : d->mAllTrackData.size() + d->mAllAlbumData.size() + d->mAllArtistData.size() + d->mAllGenreData.size();
 
     Q_ASSERT(index.isValid());
     Q_ASSERT(index.column() == 0);
-    Q_ASSERT(index.row() >= 0 && index.row() < dataCount);
     Q_ASSERT(!index.parent().isValid());
     Q_ASSERT(index.model() == this);
     Q_ASSERT(index.internalId() == 0);
+    Q_ASSERT(index.row() >= 0 && index.row() < dataCount);
 
     switch(role)
     {
@@ -145,6 +154,9 @@ QVariant DataModel::data(const QModelIndex &index, int role) const
         case ElisaUtils::Genre:
             result = d->mAllGenreData[index.row()][GenreDataType::key_type::TitleRole];
             break;
+        case ElisaUtils::Radio:
+            result = d->mAllRadiosData[index.row()][GenreDataType::key_type::TitleRole];
+            break;
         case ElisaUtils::Lyricist:
         case ElisaUtils::Composer:
         case ElisaUtils::FileName:
@@ -152,15 +164,52 @@ QVariant DataModel::data(const QModelIndex &index, int role) const
             break;
         }
         break;
-    case DatabaseInterface::ColumnsRoles::DurationRole:
+    case DataTypes::ColumnsRoles::DurationRole:
     {
-        if (d->mModelType == ElisaUtils::Track) {
+        switch (d->mModelType)
+        {
+        case ElisaUtils::Track:
+        {
             auto trackDuration = d->mAllTrackData[index.row()][TrackDataType::key_type::DurationRole].toTime();
             if (trackDuration.hour() == 0) {
                 result = trackDuration.toString(QStringLiteral("mm:ss"));
             } else {
                 result = trackDuration.toString();
             }
+            break;
+        }
+        case ElisaUtils::Album:
+        case ElisaUtils::Artist:
+        case ElisaUtils::Genre:
+        case ElisaUtils::Lyricist:
+        case ElisaUtils::Composer:
+        case ElisaUtils::FileName:
+        case ElisaUtils::Radio:
+        case ElisaUtils::Unknown:
+            break;
+        }
+        break;
+    }
+    case DataTypes::ColumnsRoles::IsSingleDiscAlbumRole:
+    {
+        switch (d->mModelType)
+        {
+        case ElisaUtils::Track:
+            result = d->mAllTrackData[index.row()][TrackDataType::key_type::IsSingleDiscAlbumRole];
+            break;
+        case ElisaUtils::Radio:
+            result = false;
+            break;
+        case ElisaUtils::Album:
+            result = d->mAllAlbumData[index.row()][AlbumDataType::key_type::IsSingleDiscAlbumRole];
+            break;
+        case ElisaUtils::Artist:
+        case ElisaUtils::Genre:
+        case ElisaUtils::Lyricist:
+        case ElisaUtils::Composer:
+        case ElisaUtils::FileName:
+        case ElisaUtils::Unknown:
+            break;
         }
         break;
     }
@@ -179,6 +228,9 @@ QVariant DataModel::data(const QModelIndex &index, int role) const
         case ElisaUtils::Genre:
             result = d->mAllGenreData[index.row()][static_cast<GenreDataType::key_type>(role)];
             break;
+        case ElisaUtils::Radio:
+            result = d->mAllRadiosData[index.row()][static_cast<TrackDataType::key_type>(role)];
+            break;
         case ElisaUtils::Lyricist:
         case ElisaUtils::Composer:
         case ElisaUtils::FileName:
@@ -232,55 +284,14 @@ bool DataModel::isBusy() const
 }
 
 void DataModel::initialize(MusicListenersManager *manager, DatabaseInterface *database,
-                           ElisaUtils::PlayListEntryType modelType)
-{
-    initializeModel(manager, database, modelType, FilterType::NoFilter);
-}
-
-void DataModel::initializeById(MusicListenersManager *manager, DatabaseInterface *database,
-                               ElisaUtils::PlayListEntryType modelType, qulonglong databaseId)
+                           ElisaUtils::PlayListEntryType modelType, ElisaUtils::FilterType filter,
+                           const QString &genre, const QString &artist, qulonglong databaseId)
 {
     d->mDatabaseId = databaseId;
-
-    initializeModel(manager, database, modelType, FilterType::FilterById);
-}
-
-void DataModel::initializeByGenre(MusicListenersManager *manager, DatabaseInterface *database,
-                                  ElisaUtils::PlayListEntryType modelType, const QString &genre)
-{
-    d->mGenre = genre;
-
-    initializeModel(manager, database, modelType, FilterType::FilterByGenre);
-}
-
-void DataModel::initializeByArtist(MusicListenersManager *manager, DatabaseInterface *database,
-                                   ElisaUtils::PlayListEntryType modelType, const QString &artist)
-{
-    d->mArtist = artist;
-
-    initializeModel(manager, database, modelType, FilterType::FilterByArtist);
-}
-
-void DataModel::initializeByGenreAndArtist(MusicListenersManager *manager, DatabaseInterface *database,
-                                           ElisaUtils::PlayListEntryType modelType, const QString &genre,
-                                           const QString &artist)
-{
     d->mGenre = genre;
     d->mArtist = artist;
 
-    initializeModel(manager, database, modelType, FilterType::FilterByGenreAndArtist);
-}
-
-void DataModel::initializeRecentlyPlayed(MusicListenersManager *manager, DatabaseInterface *database,
-                                         ElisaUtils::PlayListEntryType modelType)
-{
-    initializeModel(manager, database, modelType, FilterType::RecentlyPlayed);
-}
-
-void DataModel::initializeFrequentlyPlayed(MusicListenersManager *manager, DatabaseInterface *database,
-                                           ElisaUtils::PlayListEntryType modelType)
-{
-    initializeModel(manager, database, modelType, FilterType::FrequentlyPlayed);
+    initializeModel(manager, database, modelType, filter);
 }
 
 void DataModel::setBusy(bool value)
@@ -300,7 +311,7 @@ void DataModel::initializeModel(MusicListenersManager *manager, DatabaseInterfac
     d->mFilterType = type;
 
     if (manager) {
-        manager->connectModel(&d->mDataLoader);
+        manager->connectModel(d->mDataLoader);
     }
 
     if (manager) {
@@ -313,35 +324,35 @@ void DataModel::initializeModel(MusicListenersManager *manager, DatabaseInterfac
 
     switch(d->mFilterType)
     {
-    case NoFilter:
+    case ElisaUtils::NoFilter:
         connect(this, &DataModel::needData,
-                &d->mDataLoader, &ModelDataLoader::loadData);
+                d->mDataLoader, &ModelDataLoader::loadData);
         break;
-    case FilterById:
+    case ElisaUtils::FilterById:
         connect(this, &DataModel::needDataById,
-                &d->mDataLoader, &ModelDataLoader::loadDataByAlbumId);
+                d->mDataLoader, &ModelDataLoader::loadDataByAlbumId);
         break;
-    case FilterByGenre:
+    case ElisaUtils::FilterByGenre:
         connect(this, &DataModel::needDataByGenre,
-                &d->mDataLoader, &ModelDataLoader::loadDataByGenre);
+                d->mDataLoader, &ModelDataLoader::loadDataByGenre);
         break;
-    case FilterByArtist:
+    case ElisaUtils::FilterByArtist:
         connect(this, &DataModel::needDataByArtist,
-                &d->mDataLoader, &ModelDataLoader::loadDataByArtist);
+                d->mDataLoader, &ModelDataLoader::loadDataByArtist);
         break;
-    case FilterByGenreAndArtist:
+    case ElisaUtils::FilterByGenreAndArtist:
         connect(this, &DataModel::needDataByGenreAndArtist,
-                &d->mDataLoader, &ModelDataLoader::loadDataByGenreAndArtist);
+                d->mDataLoader, &ModelDataLoader::loadDataByGenreAndArtist);
         break;
-    case RecentlyPlayed:
+    case ElisaUtils::FilterByRecentlyPlayed:
         connect(this, &DataModel::needRecentlyPlayedData,
-                &d->mDataLoader, &ModelDataLoader::loadRecentlyPlayedData);
+                d->mDataLoader, &ModelDataLoader::loadRecentlyPlayedData);
         break;
-    case FrequentlyPlayed:
+    case ElisaUtils::FilterByFrequentlyPlayed:
         connect(this, &DataModel::needFrequentlyPlayedData,
-                &d->mDataLoader, &ModelDataLoader::loadFrequentlyPlayedData);
+                d->mDataLoader, &ModelDataLoader::loadFrequentlyPlayedData);
         break;
-    case Unknown:
+    case ElisaUtils::UnknownFilter:
         break;
     }
 
@@ -354,38 +365,39 @@ void DataModel::askModelData()
 {
     switch(d->mFilterType)
     {
-    case NoFilter:
+    case ElisaUtils::NoFilter:
         Q_EMIT needData(d->mModelType);
         break;
-    case FilterById:
+    case ElisaUtils::FilterById:
         Q_EMIT needDataById(d->mModelType, d->mDatabaseId);
         break;
-    case FilterByGenre:
+    case ElisaUtils::FilterByGenre:
         Q_EMIT needDataByGenre(d->mModelType, d->mGenre);
         break;
-    case FilterByArtist:
+    case ElisaUtils::FilterByArtist:
         Q_EMIT needDataByArtist(d->mModelType, d->mArtist);
         break;
-    case FilterByGenreAndArtist:
+    case ElisaUtils::FilterByGenreAndArtist:
         Q_EMIT needDataByGenreAndArtist(d->mModelType, d->mGenre, d->mArtist);
         break;
-    case RecentlyPlayed:
+    case ElisaUtils::FilterByRecentlyPlayed:
         Q_EMIT needRecentlyPlayedData(d->mModelType);
         break;
-    case FrequentlyPlayed:
+    case ElisaUtils::FilterByFrequentlyPlayed:
         Q_EMIT needFrequentlyPlayedData(d->mModelType);
         break;
-    case Unknown:
+    case ElisaUtils::UnknownFilter:
         break;
     }
 }
 
-int DataModel::trackIndexFromId(qulonglong id) const
+int DataModel::indexFromId(qulonglong id) const
 {
     int result;
+    DataModel::ListTrackDataType mAllData = d->mModelType == ElisaUtils::Radio ? d->mAllRadiosData: d->mAllTrackData;
 
-    for (result = 0; result < d->mAllTrackData.size(); ++result) {
-        if (d->mAllTrackData[result].databaseId() == id) {
+    for (result = 0; result < mAllData.size(); ++result) {
+        if (mAllData[result].databaseId() == id) {
             return result;
         }
     }
@@ -397,34 +409,42 @@ int DataModel::trackIndexFromId(qulonglong id) const
 
 void DataModel::connectModel(DatabaseInterface *database)
 {
-    d->mDataLoader.setDatabase(database);
+    d->mDataLoader->setDatabase(database);
 
-    connect(&d->mDataLoader, &ModelDataLoader::allTracksData,
+    connect(d->mDataLoader, &ModelDataLoader::allTracksData,
             this, &DataModel::tracksAdded);
-    connect(&d->mDataLoader, &ModelDataLoader::allAlbumsData,
+    connect(d->mDataLoader, &ModelDataLoader::allRadiosData,
+            this, &DataModel::radiosAdded);
+    connect(d->mDataLoader, &ModelDataLoader::allAlbumsData,
             this, &DataModel::albumsAdded);
-    connect(&d->mDataLoader, &ModelDataLoader::allArtistsData,
+    connect(d->mDataLoader, &ModelDataLoader::allArtistsData,
             this, &DataModel::artistsAdded);
-    connect(&d->mDataLoader, &ModelDataLoader::allGenresData,
+    connect(d->mDataLoader, &ModelDataLoader::allGenresData,
             this, &DataModel::genresAdded);
-    connect(&d->mDataLoader, &ModelDataLoader::genresAdded,
+    connect(d->mDataLoader, &ModelDataLoader::genresAdded,
             this, &DataModel::genresAdded);
-    connect(&d->mDataLoader, &ModelDataLoader::albumsAdded,
+    connect(d->mDataLoader, &ModelDataLoader::albumsAdded,
             this, &DataModel::albumsAdded);
-    connect(&d->mDataLoader, &ModelDataLoader::albumModified,
+    connect(d->mDataLoader, &ModelDataLoader::albumModified,
             this, &DataModel::albumModified);
-    connect(&d->mDataLoader, &ModelDataLoader::albumRemoved,
+    connect(d->mDataLoader, &ModelDataLoader::albumRemoved,
             this, &DataModel::albumRemoved);
-    connect(&d->mDataLoader, &ModelDataLoader::tracksAdded,
+    connect(d->mDataLoader, &ModelDataLoader::tracksAdded,
             this, &DataModel::tracksAdded);
-    connect(&d->mDataLoader, &ModelDataLoader::trackModified,
+    connect(d->mDataLoader, &ModelDataLoader::trackModified,
             this, &DataModel::trackModified);
-    connect(&d->mDataLoader, &ModelDataLoader::trackRemoved,
+    connect(d->mDataLoader, &ModelDataLoader::trackRemoved,
             this, &DataModel::trackRemoved);
-    connect(&d->mDataLoader, &ModelDataLoader::artistsAdded,
+    connect(d->mDataLoader, &ModelDataLoader::artistsAdded,
             this, &DataModel::artistsAdded);
-    connect(&d->mDataLoader, &ModelDataLoader::artistRemoved,
+    connect(d->mDataLoader, &ModelDataLoader::artistRemoved,
             this, &DataModel::artistRemoved);
+    connect(d->mDataLoader, &ModelDataLoader::radioAdded,
+            this, &DataModel::radioAdded);
+    connect(d->mDataLoader, &ModelDataLoader::radioModified,
+            this, &DataModel::radioModified);
+    connect(d->mDataLoader, &ModelDataLoader::radioRemoved,
+            this, &DataModel::radioRemoved);
 }
 
 void DataModel::tracksAdded(ListTrackDataType newData)
@@ -437,9 +457,9 @@ void DataModel::tracksAdded(ListTrackDataType newData)
         return;
     }
 
-    if (d->mFilterType == FilterById && !d->mAllTrackData.isEmpty()) {
+    if (d->mFilterType == ElisaUtils::FilterById && !d->mAllTrackData.isEmpty()) {
         for (const auto &newTrack : newData) {
-            auto trackIndex = trackIndexFromId(newTrack.databaseId());
+            auto trackIndex = indexFromId(newTrack.databaseId());
 
             if (trackIndex != -1) {
                 continue;
@@ -488,6 +508,67 @@ void DataModel::tracksAdded(ListTrackDataType newData)
     }
 }
 
+void DataModel::radiosAdded(ListRadioDataType newData)
+{
+    if (newData.isEmpty() && d->mModelType == ElisaUtils::Radio) {
+        setBusy(false);
+    }
+
+    if (newData.isEmpty() || d->mModelType != ElisaUtils::Radio) {
+        return;
+    }
+
+    if (d->mFilterType == ElisaUtils::FilterById && !d->mAllRadiosData.isEmpty()) {
+        for (const auto &newTrack : newData) {
+            auto trackIndex = indexFromId(newTrack.databaseId());
+
+            if (trackIndex != -1) {
+                continue;
+            }
+
+            bool trackInserted = false;
+            for (int trackIndex = 0; trackIndex < d->mAllRadiosData.count(); ++trackIndex) {
+                const auto &oneTrack = d->mAllRadiosData[trackIndex];
+
+                if (oneTrack.trackNumber() > newTrack.trackNumber()) {
+                    beginInsertRows({}, trackIndex, trackIndex);
+                    d->mAllRadiosData.insert(trackIndex, newTrack);
+                    endInsertRows();
+
+                    if (d->mAllRadiosData.size() == 1) {
+                        setBusy(false);
+                    }
+
+                    trackInserted = true;
+                    break;
+                }
+            }
+
+            if (!trackInserted) {
+                beginInsertRows({}, d->mAllRadiosData.count(), d->mAllRadiosData.count());
+                d->mAllRadiosData.insert(d->mAllRadiosData.count(), newTrack);
+                endInsertRows();
+
+                if (d->mAllRadiosData.size() == 1) {
+                    setBusy(false);
+                }
+            }
+        }
+    } else {
+        if (d->mAllRadiosData.isEmpty()) {
+            beginInsertRows({}, 0, newData.size() - 1);
+            d->mAllRadiosData.swap(newData);
+            endInsertRows();
+
+            setBusy(false);
+        } else {
+            beginInsertRows({}, d->mAllRadiosData.size(), d->mAllRadiosData.size() + newData.size() - 1);
+            d->mAllRadiosData.append(newData);
+            endInsertRows();
+        }
+    }
+}
+
 void DataModel::trackModified(const TrackDataType &modifiedTrack)
 {
     if (d->mModelType != ElisaUtils::Track) {
@@ -499,7 +580,7 @@ void DataModel::trackModified(const TrackDataType &modifiedTrack)
             return;
         }
 
-        auto trackIndex = trackIndexFromId(modifiedTrack.databaseId());
+        auto trackIndex = indexFromId(modifiedTrack.databaseId());
 
         if (trackIndex == -1) {
             return;
@@ -525,6 +606,22 @@ void DataModel::trackModified(const TrackDataType &modifiedTrack)
     }
 }
 
+void DataModel::radioModified(const TrackDataType &modifiedRadio)
+{
+    if (d->mModelType != ElisaUtils::Radio) {
+        return;
+    }
+
+    auto trackIndex = indexFromId(modifiedRadio.databaseId());
+
+    if (trackIndex == -1) {
+        return;
+    }
+
+    d->mAllRadiosData[trackIndex] = modifiedRadio;
+    Q_EMIT dataChanged(index(trackIndex, 0), index(trackIndex, 0));
+}
+
 void DataModel::trackRemoved(qulonglong removedTrackId)
 {
     if (d->mModelType != ElisaUtils::Track) {
@@ -532,7 +629,7 @@ void DataModel::trackRemoved(qulonglong removedTrackId)
     }
 
     if (!d->mAlbumTitle.isEmpty() && !d->mAlbumArtist.isEmpty()) {
-        auto trackIndex = trackIndexFromId(removedTrackId);
+        auto trackIndex = indexFromId(removedTrackId);
 
         if (trackIndex == -1) {
             return;
@@ -557,6 +654,49 @@ void DataModel::trackRemoved(qulonglong removedTrackId)
     }
 }
 
+void DataModel::radioRemoved(qulonglong removedRadioId)
+{
+    if (d->mModelType != ElisaUtils::Radio) {
+        return;
+    }
+
+
+    auto itRadio = std::find_if(d->mAllRadiosData.begin(), d->mAllRadiosData.end(),
+                                [removedRadioId](auto track) {return track.databaseId() == removedRadioId;});
+
+    if (itRadio == d->mAllRadiosData.end()) {
+        return;
+    }
+
+    auto position = itRadio - d->mAllRadiosData.begin();
+
+    beginRemoveRows({}, position, position);
+    d->mAllRadiosData.erase(itRadio);
+    endRemoveRows();
+}
+
+void DataModel::radioAdded(const DataModel::TrackDataType radioData)
+{
+    if (d->mModelType != ElisaUtils::Radio) {
+        return;
+    }
+
+    ListRadioDataType list;
+    list.append(radioData);
+    radiosAdded(list);
+}
+
+void DataModel::removeRadios()
+{
+    if (d->mModelType != ElisaUtils::Radio) {
+        return;
+    }
+
+    beginRemoveRows({}, 0, d->mAllRadiosData.size());
+    d->mAllRadiosData.clear();
+    endRemoveRows();
+}
+
 void DataModel::genresAdded(DataModel::ListGenreDataType newData)
 {
     if (newData.isEmpty() && d->mModelType == ElisaUtils::Genre) {
