@@ -21,7 +21,6 @@
 
 #include "elisaLib_export.h"
 
-#include "musicaudiotrack.h"
 #include "filescanner.h"
 #include "elisautils.h"
 
@@ -65,8 +64,6 @@ public:
 
     bool canGoBack() const;
 
-    Q_INVOKABLE MusicAudioTrack loadMetaDataFromUrl(const QUrl &url);
-
     bool sortedAscending() const;
 
     void setSourceModel(QAbstractItemModel *sourceModel) override;
@@ -85,8 +82,6 @@ public Q_SLOTS:
 
     void sortModel(Qt::SortOrder order);
 
-    void replaceAndPlayOfUrl(const QUrl &fileUrl);
-
 Q_SIGNALS:
 
     void filesToEnqueue(const ElisaUtils::EntryDataList &newFiles,
@@ -102,8 +97,6 @@ Q_SIGNALS:
 
     void sortedAscendingChanged();
 
-    void loadPlayListFromUrl(const QUrl &playListUrl);
-
 protected:
 
     bool filterAcceptsRow(int source_row, const QModelIndex &source_parent) const override;
@@ -116,8 +109,6 @@ private:
 
     FileScanner mFileScanner;
 
-    QMimeDatabase mMimeDb;
-
     QString mFilterText;
 
     QRegularExpression mFilterExpression;
