@@ -39,7 +39,6 @@
 #include "file/filelistener.h"
 #include "file/localfilelisting.h"
 #include "trackslistener.h"
-#include "notificationitem.h"
 #include "elisaapplication.h"
 #include "elisa_settings.h"
 #include "modeldataloader.h"
@@ -112,10 +111,13 @@ public:
 MusicListenersManager::MusicListenersManager(QObject *parent)
     : QObject(parent), d(std::make_unique<MusicListenersManagerPrivate>())
 {
-    d->mListenerThread.start();
-    d->mDatabaseThread.start();
+    connect(&d->mDatabaseInterface, &DatabaseInterface::tracksAdded,
+            this, &MusicListenersManager::increaseImportedTracksCount);
 
-    d->mDatabaseInterface.moveToThread(&d->mDatabaseThread);
+#if defined KF5Baloo_FOUND && KF5Baloo_FOUND
+    connect(&d->mBalooDetector, &BalooDetector::balooAvailabilityChanged,
+            this, &MusicListenersManager::balooAvailabilityChanged);
+#endif
 
     connect(&d->mDatabaseInterface, &DatabaseInterface::requestsInitDone,
             this, &MusicListenersManager::databaseReady);
@@ -124,6 +126,17 @@ MusicListenersManager::MusicListenersManager(QObject *parent)
     connect(&d->mDatabaseInterface, &DatabaseInterface::cleanedDatabase,
             this, &MusicListenersManager::cleanedDatabase);
 
+    connect(QCoreApplication::instance(), &QCoreApplication::aboutToQuit,
+            this, &MusicListenersManager::applicationAboutToQuit);
+
+    connect(&d->mConfigFileWatcher, &QFileSystemWatcher::fileChanged,
+            this, &MusicListenersManager::configChanged);
+
+    d->mListenerThread.start();
+    d->mDatabaseThread.start();
+
+    d->mDatabaseInterface.moveToThread(&d->mDatabaseThread);
+
     const auto &localDataPaths = QStandardPaths::standardLocations(QStandardPaths::AppDataLocation);
     auto databaseFileName = QString();
     if (!localDataPaths.isEmpty()) {
@@ -135,36 +148,6 @@ MusicListenersManager::MusicListenersManager(QObject *parent)
     QMetaObject::invokeMethod(&d->mDatabaseInterface, "init", Qt::QueuedConnection,
                               Q_ARG(QString, QStringLiteral("listeners")), Q_ARG(QString, databaseFileName));
 
-    connect(QCoreApplication::instance(), &QCoreApplication::aboutToQuit,
-            this, &MusicListenersManager::applicationAboutToQuit);
-
-    connect(Elisa::ElisaConfiguration::self(), &Elisa::ElisaConfiguration::configChanged,
-            this, &MusicListenersManager::configChanged);
-
-    connect(&d->mConfigFileWatcher, &QFileSystemWatcher::fileChanged,
-            this, &MusicListenersManager::configChanged);
-
-    auto initialRootPath = Elisa::ElisaConfiguration::rootPath();
-    if (initialRootPath.isEmpty()) {
-        auto systemMusicPaths = QStandardPaths::standardLocations(QStandardPaths::MusicLocation);
-        for (const auto &musicPath : qAsConst(systemMusicPaths)) {
-            initialRootPath.push_back(musicPath);
-        }
-
-        Elisa::ElisaConfiguration::setRootPath(initialRootPath);
-        Elisa::ElisaConfiguration::self()->save();
-    }
-
-    d->mConfigFileWatcher.addPath(Elisa::ElisaConfiguration::self()->config()->name());
-
-    connect(&d->mDatabaseInterface, &DatabaseInterface::tracksAdded,
-            this, &MusicListenersManager::increaseImportedTracksCount);
-
-#if defined KF5Baloo_FOUND && KF5Baloo_FOUND
-    connect(&d->mBalooDetector, &BalooDetector::balooAvailabilityChanged,
-            this, &MusicListenersManager::balooAvailabilityChanged);
-#endif
-
     qCInfo(orgKdeElisaIndexersManager) << "Local file system indexer is inactive";
     qCInfo(orgKdeElisaIndexersManager) << "Baloo indexer is unavailable";
     qCInfo(orgKdeElisaIndexersManager) << "Baloo indexer is inactive";
@@ -185,6 +168,7 @@ void MusicListenersManager::subscribeForTracks(MediaPlayList *client)
     connect(d->mTracksListener.get(), &TracksListener::trackHasBeenRemoved, client, &MediaPlayList::trackRemoved);
     connect(d->mTracksListener.get(), &TracksListener::tracksListAdded, client, &MediaPlayList::tracksListAdded);
     connect(client, &MediaPlayList::newEntryInList, d->mTracksListener.get(), &TracksListener::newEntryInList);
+    connect(client, &MediaPlayList::newUrlInList, d->mTracksListener.get(), &TracksListener::newUrlInList);
     connect(client, &MediaPlayList::newTrackByNameInList, d->mTracksListener.get(), &TracksListener::trackByNameInList);
 }
 
@@ -198,6 +182,11 @@ ElisaApplication *MusicListenersManager::elisaApplication() const
     return d->mElisaApplication;
 }
 
+TracksListener *MusicListenersManager::tracksListener() const
+{
+    return d->mTracksListener.get();
+}
+
 bool MusicListenersManager::indexerBusy() const
 {
     return d->mIndexerBusy;
@@ -230,6 +219,19 @@ bool MusicListenersManager::androidIndexerAvailable() const
 
 void MusicListenersManager::databaseReady()
 {
+    auto initialRootPath = Elisa::ElisaConfiguration::rootPath();
+    if (initialRootPath.isEmpty()) {
+        auto systemMusicPaths = QStandardPaths::standardLocations(QStandardPaths::MusicLocation);
+        for (const auto &musicPath : qAsConst(systemMusicPaths)) {
+            initialRootPath.push_back(musicPath);
+        }
+
+        Elisa::ElisaConfiguration::setRootPath(initialRootPath);
+        Elisa::ElisaConfiguration::self()->save();
+    }
+
+    d->mConfigFileWatcher.addPath(Elisa::ElisaConfiguration::self()->config()->name());
+
     configChanged();
 }
 
@@ -278,6 +280,26 @@ void MusicListenersManager::playBackError(const QUrl &sourceInError, QMediaPlaye
     }
 }
 
+void MusicListenersManager::deleteElementById(ElisaUtils::PlayListEntryType entryType, qulonglong databaseId)
+{
+    switch(entryType)
+    {
+    case ElisaUtils::Radio:
+        QMetaObject::invokeMethod(&d->mDatabaseInterface, "removeRadio", Qt::QueuedConnection,
+                                  Q_ARG(qulonglong, databaseId));
+        break;
+    case ElisaUtils::Album:
+    case ElisaUtils::Artist:
+    case ElisaUtils::Genre:
+    case ElisaUtils::Lyricist:
+    case ElisaUtils::Composer:
+    case ElisaUtils::Track:
+    case ElisaUtils::FileName:
+    case ElisaUtils::Unknown:
+        break;
+    }
+}
+
 void MusicListenersManager::connectModel(ModelDataLoader *dataLoader)
 {
     dataLoader->moveToThread(&d->mDatabaseThread);
@@ -306,14 +328,13 @@ void MusicListenersManager::configChanged()
 
     if (!d->mBalooIndexerActive && !d->mFileSystemIndexerActive) {
         testBalooIndexerAvailability();
-    } else {
-        if (d->mBalooIndexerActive) {
+    }
+    if (d->mBalooIndexerActive) {
 #if defined KF5Baloo_FOUND && KF5Baloo_FOUND
-            QMetaObject::invokeMethod(d->mBalooListener.fileListing(), "init", Qt::QueuedConnection);
+        QMetaObject::invokeMethod(d->mBalooListener.fileListing(), "init", Qt::QueuedConnection);
 #endif
-        } else if (d->mFileSystemIndexerActive) {
-            QMetaObject::invokeMethod(d->mFileListener.fileListing(), "init", Qt::QueuedConnection);
-        }
+    } else if (d->mFileSystemIndexerActive) {
+        QMetaObject::invokeMethod(d->mFileListener.fileListing(), "init", Qt::QueuedConnection);
     }
 
 #if defined UPNPQT_FOUND && UPNPQT_FOUND
@@ -336,22 +357,14 @@ void MusicListenersManager::configChanged()
                 this, &MusicListenersManager::monitorEndingListeners);
         connect(d->mAndroidMusicListener.get(), &AndroidMusicListener::clearDatabase,
                 &d->mDatabaseInterface, &DatabaseInterface::removeAllTracksFromSource);
-        connect(d->mAndroidMusicListener.get(), &AndroidMusicListener::newNotification,
-                this, &MusicListenersManager::newNotification);
-        connect(d->mAndroidMusicListener.get(), &AndroidMusicListener::closeNotification,
-                this, &MusicListenersManager::closeNotification);
     }
 #endif
 }
 
-void MusicListenersManager::increaseImportedTracksCount(const DatabaseInterface::ListTrackDataType &allTracks)
+void MusicListenersManager::increaseImportedTracksCount(const DataTypes::ListTrackDataType &allTracks)
 {
     d->mImportedTracksCount += allTracks.size();
 
-    //if (d->mImportedTracksCount >= 4) {
-        Q_EMIT closeNotification(QStringLiteral("notEnoughTracks"));
-    //}
-
     Q_EMIT importedTracksCountChanged();
 }
 
@@ -370,24 +383,6 @@ void MusicListenersManager::monitorStartingListeners()
 
 void MusicListenersManager::monitorEndingListeners()
 {
-    /*if (d->mImportedTracksCount < 4 && d->mElisaApplication) {
-        NotificationItem notEnoughTracks;
-
-        notEnoughTracks.setNotificationId(QStringLiteral("notEnoughTracks"));
-
-        notEnoughTracks.setTargetObject(this);
-
-        notEnoughTracks.setMessage(i18nc("No track found message", "No track have been found"));
-
-        auto configureAction = d->mElisaApplication->action(QStringLiteral("options_configure"));
-
-        notEnoughTracks.setMainButtonText(configureAction->text());
-        notEnoughTracks.setMainButtonIconName(configureAction->icon().name());
-        notEnoughTracks.setMainButtonMethodName(QStringLiteral("showConfiguration"));
-
-        Q_EMIT newNotification(notEnoughTracks);
-    }*/
-
     d->mIndexerBusy = false;
     Q_EMIT indexerBusyChanged();
 }
@@ -449,12 +444,6 @@ void MusicListenersManager::startLocalFileSystemIndexing()
             this, &MusicListenersManager::monitorStartingListeners);
     connect(&d->mFileListener, &FileListener::indexingFinished,
             this, &MusicListenersManager::monitorEndingListeners);
-    connect(&d->mFileListener, &FileListener::newNotification,
-            this, &MusicListenersManager::newNotification);
-    connect(&d->mFileListener, &FileListener::closeNotification,
-            this, &MusicListenersManager::closeNotification);
-
-    QMetaObject::invokeMethod(d->mFileListener.fileListing(), "init", Qt::QueuedConnection);
 
     qCInfo(orgKdeElisaIndexersManager) << "Local file system indexer is active";
 
@@ -475,12 +464,6 @@ void MusicListenersManager::startBalooIndexing()
             this, &MusicListenersManager::monitorEndingListeners);
     connect(&d->mBalooListener, &BalooListener::clearDatabase,
             &d->mDatabaseInterface, &DatabaseInterface::clearData);
-    connect(&d->mBalooListener, &BalooListener::newNotification,
-            this, &MusicListenersManager::newNotification);
-    connect(&d->mBalooListener, &BalooListener::closeNotification,
-            this, &MusicListenersManager::closeNotification);
-
-    QMetaObject::invokeMethod(d->mBalooListener.fileListing(), "init", Qt::QueuedConnection);
 
     qCInfo(orgKdeElisaIndexersManager) << "Baloo indexer is active";
 
