@@ -20,19 +20,20 @@
 
 #include "elisaLib_export.h"
 
-#include "notificationitem.h"
-#include "databaseinterface.h"
+#include "datatypes.h"
+#include "elisautils.h"
 
 #include <QObject>
 #include <QMediaPlayer>
 
 #include <memory>
 
+class DatabaseInterface;
 class MusicListenersManagerPrivate;
 class MediaPlayList;
-class NotificationItem;
 class ElisaApplication;
 class ModelDataLoader;
+class TracksListener;
 
 class ELISALIB_EXPORT MusicListenersManager : public QObject
 {
@@ -52,6 +53,10 @@ class ELISALIB_EXPORT MusicListenersManager : public QObject
                WRITE setElisaApplication
                NOTIFY elisaApplicationChanged)
 
+    Q_PROPERTY(TracksListener* tracksListener
+               READ tracksListener
+               NOTIFY tracksListenerChanged)
+
     Q_PROPERTY(bool indexerBusy
                READ indexerBusy
                NOTIFY indexerBusyChanged)
@@ -90,6 +95,8 @@ public:
 
     ElisaApplication* elisaApplication() const;
 
+    TracksListener* tracksListener() const;
+
     bool indexerBusy() const;
 
     bool fileSystemIndexerActive() const;
@@ -108,15 +115,13 @@ Q_SIGNALS:
 
     void applicationIsTerminating();
 
+    void tracksListenerChanged();
+
     void importedTracksCountChanged();
 
-    void newNotification(NotificationItem notification);
-
-    void closeNotification(QString notificationId);
-
     void elisaApplicationChanged();
 
-    void removeTracksInError(QList<QUrl> tracks);
+    void removeTracksInError(const QList<QUrl> &tracks);
 
     void displayTrackError(const QString &fileName);
 
@@ -146,6 +151,8 @@ public Q_SLOTS:
 
     void playBackError(const QUrl &sourceInError, QMediaPlayer::Error playerError);
 
+    void deleteElementById(ElisaUtils::PlayListEntryType entryType, qulonglong databaseId);
+
     void connectModel(ModelDataLoader *dataLoader);
 
     void resetMusicData();
@@ -154,7 +161,7 @@ private Q_SLOTS:
 
     void configChanged();
 
-    void increaseImportedTracksCount(const DatabaseInterface::ListTrackDataType &allTracks);
+    void increaseImportedTracksCount(const DataTypes::ListTrackDataType &allTracks);
 
     void decreaseImportedTracksCount();
 
