@@ -21,7 +21,7 @@
 #include "elisaLib_export.h"
 
 #include "databaseinterface.h"
-#include "musicaudiotrack.h"
+#include "datatypes.h"
 #include "elisautils.h"
 
 #include <QObject>
@@ -36,9 +36,9 @@ class ELISALIB_EXPORT TracksListener : public QObject
 
 public:
 
-    using ListTrackDataType = DatabaseInterface::ListTrackDataType;
+    using ListTrackDataType = DataTypes::ListTrackDataType;
 
-    using TrackDataType = DatabaseInterface::TrackDataType;
+    using TrackDataType = DataTypes::TrackDataType;
 
     explicit TracksListener(DatabaseInterface *database, QObject *parent = nullptr);
 
@@ -69,14 +69,15 @@ public Q_SLOTS:
                         const QString &entryTitle,
                         ElisaUtils::PlayListEntryType databaseIdType);
 
+    void trackByFileNameInList(ElisaUtils::PlayListEntryType databaseIdType, const QUrl &fileName);
+
+    void newUrlInList(const QUrl &entryUrl,
+                      ElisaUtils::PlayListEntryType databaseIdType);
+
 private:
 
-    MusicAudioTrack scanOneFile(const QUrl &scanFile);
-
     void newArtistInList(qulonglong newDatabaseId, const QString &artist);
 
-    void trackByFileNameInList(const QUrl &fileName);
-
     void newAlbumInList(qulonglong newDatabaseId,
                         const QString &entryTitle);
 
